@@ -23,27 +23,38 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gdk/gdk.h>
 
 #include "gpm-common.h"
 
+/**
+ * gpm_warning_beep
+ **/
+void
+gpm_warning_beep (void)
+{
+	GdkDisplay *display = gdk_display_get_default ();
+	gdk_display_beep (display);
+}
+
 /**
  * gpm_get_timestring:
- * @time: The time value to convert in seconds
+ * @time_secs: The time value to convert in seconds
  * @cookie: The cookie we are looking for
  *
  * Returns a localised timestring
  *
  * Return value: The time string, e.g. "2 hours 3 minutes"
  **/
-char *
-gpm_get_timestring (int time)
+gchar *
+gpm_get_timestring (guint time_secs)
 {
 	char* timestring = NULL;
 	gint  hours;
 	gint  minutes;
 
 	/* Add 0.5 to do rounding */
-	minutes = (int) ( ( time / 60.0 ) + 0.5 );
+	minutes = (int) ( ( time_secs / 60.0 ) + 0.5 );
 
 	if (minutes == 0) {
 		timestring = g_strdup_printf (_("Unknown time"));
