@@ -40,6 +40,7 @@
 #include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-hal.h"
+#include "gpm-hal-power.h"
 #include "gpm-marshal.h"
 #include "gpm-debug.h"
 
@@ -54,6 +55,7 @@ static void     gpm_hal_monitor_finalize   (GObject	       *object);
 struct GpmHalMonitorPrivate
 {
 	GpmHal			*hal;
+	GpmHalPower		*hal_power;
 };
 
 enum {
@@ -170,11 +172,11 @@ monitor_change_on_ac (GpmHalMonitor *monitor,
  */
 static void
 emit_button_pressed (GpmHalMonitor *monitor,
-		     const char	   *udi,
-		     const char	   *details)
+		     const gchar   *udi,
+		     const gchar   *details)
 {
-	char	   *button_name = NULL;
-	gboolean    value;
+	gchar *button_name = NULL;
+	gboolean value;
 
 	g_return_if_fail (udi != NULL);
 	g_return_if_fail (details != NULL);
@@ -187,8 +189,6 @@ emit_button_pressed (GpmHalMonitor *monitor,
 		button_name = g_strdup (details);
 	}
 
-	g_return_if_fail (button_name != NULL);
-
 	/* Buttons without state should default to true. */
 	value = TRUE;
 	/* we need to get the button state for lid buttons */
@@ -215,9 +215,9 @@ emit_button_pressed (GpmHalMonitor *monitor,
  * changed, and we have we have subscribed to changes for that device.
  */
 static void
-hal_device_property_modified_cb (GpmHal       *hal,
-				 const char    *udi,
-				 const char    *key,
+hal_device_property_modified_cb (GpmHal        *hal,
+				 const gchar   *udi,
+				 const gchar   *key,
 				 gboolean       is_added,
 				 gboolean       is_removed,
 				 gboolean       finally,
@@ -232,7 +232,7 @@ hal_device_property_modified_cb (GpmHal       *hal,
 	}
 
 	if (strcmp (key, "ac_adapter.present") == 0) {
-		gboolean on_ac = gpm_hal_is_on_ac (monitor->priv->hal);
+		gboolean on_ac = gpm_hal_power_is_on_ac (monitor->priv->hal_power);
 		monitor_change_on_ac (monitor, on_ac);
 		return;
 	}
@@ -261,9 +261,9 @@ hal_device_property_modified_cb (GpmHal       *hal,
  */
 static void
 hal_device_condition_cb (GpmHal        *hal,
-			 const char    *udi,
-			 const char    *condition,
-			 const char    *details,
+			 const gchar   *udi,
+			 const gchar   *condition,
+			 const gchar   *details,
 			 GpmHalMonitor *monitor)
 {
 	gpm_debug ("udi=%s, condition=%s, details=%s", udi, condition, details);
@@ -280,7 +280,7 @@ hal_device_condition_cb (GpmHal        *hal,
  */
 static void
 watch_add_battery (GpmHalMonitor *monitor,
-		   const char    *udi)
+		   const gchar   *udi)
 {
 	gpm_hal_device_watch_propery_modified (monitor->priv->hal, udi);
 
@@ -295,7 +295,7 @@ watch_add_battery (GpmHalMonitor *monitor,
  */
 static void
 watch_add_button (GpmHalMonitor *monitor,
-		  const char    *udi)
+		  const gchar   *udi)
 {
 	gpm_hal_device_watch_condition (monitor->priv->hal, udi);
 	gpm_hal_device_watch_propery_modified (monitor->priv->hal, udi);
@@ -308,7 +308,7 @@ watch_add_button (GpmHalMonitor *monitor,
  */
 static void
 watch_add_ac_adapter (GpmHalMonitor *monitor,
-		      const char    *udi)
+		      const gchar   *udi)
 {
 	gpm_hal_device_watch_propery_modified (monitor->priv->hal, udi);
 }
@@ -322,7 +322,7 @@ watch_add_ac_adapter (GpmHalMonitor *monitor,
  */
 static void
 hal_device_removed_cb (GpmHal        *hal,
-		       const char    *udi,
+		       const gchar   *udi,
 		       GpmHalMonitor *monitor)
 {
 	gpm_debug ("udi=%s", udi);
@@ -342,8 +342,8 @@ hal_device_removed_cb (GpmHal        *hal,
  */
 static void
 hal_new_capability_cb (GpmHal        *hal,
-		       const char    *udi,
-		       const char    *capability,
+		       const gchar   *udi,
+		       const gchar   *capability,
 		       GpmHalMonitor *monitor)
 {
 	gpm_debug ("udi=%s, capability=%s", udi, capability);
@@ -476,6 +476,7 @@ gpm_hal_monitor_init (GpmHalMonitor *monitor)
 	monitor->priv = GPM_HAL_MONITOR_GET_PRIVATE (monitor);
 
 	monitor->priv->hal = gpm_hal_new ();
+	monitor->priv->hal_power = gpm_hal_power_new ();
 
 	g_signal_connect (monitor->priv->hal, "device-removed",
 			  G_CALLBACK (hal_device_removed_cb), monitor);
@@ -507,6 +508,7 @@ gpm_hal_monitor_finalize (GObject *object)
 	g_return_if_fail (monitor->priv != NULL);
 
 	g_object_unref (monitor->priv->hal);
+	g_object_unref (monitor->priv->hal_power);
 
 	G_OBJECT_CLASS (gpm_hal_monitor_parent_class)->finalize (object);
 }
