@@ -35,6 +35,7 @@
 #include "gpm-debug.h"
 
 #include "gpm-info-data.h"
+#include "gpm-graph-widget.h"
 
 static void     gpm_info_data_class_init (GpmInfoDataClass *klass);
 static void     gpm_info_data_init       (GpmInfoData      *info_data);
@@ -45,8 +46,8 @@ static void     gpm_info_data_finalize   (GObject		 	*object);
 struct GpmInfoDataPrivate
 {
 	GPtrArray		*array;		/* the data array */
-	int			 max_points;	/* when we should simplify data */
-	int			 max_time;	/* truncate after this */
+	guint			 max_points;	/* when we should simplify data */
+	guint			 max_time;	/* truncate after this */
 };
 
 G_DEFINE_TYPE (GpmInfoData, gpm_info_data, G_TYPE_OBJECT)
@@ -57,7 +58,7 @@ G_DEFINE_TYPE (GpmInfoData, gpm_info_data, G_TYPE_OBJECT)
  * @max_points: The maximum number of points to show on the graph
  */
 void
-gpm_info_data_set_max_points (GpmInfoData *info_data, int max_points)
+gpm_info_data_set_max_points (GpmInfoData *info_data, guint max_points)
 {
 	info_data->priv->max_points = max_points;
 }
@@ -68,7 +69,7 @@ gpm_info_data_set_max_points (GpmInfoData *info_data, int max_points)
  * @max_points: The maximum number of points to show on the graph
  */
 void
-gpm_info_data_set_max_time (GpmInfoData *info_data, int max_time)
+gpm_info_data_set_max_time (GpmInfoData *info_data, guint max_time)
 {
 	if (max_time > 10 * 60) {
 		info_data->priv->max_time = max_time;
@@ -108,7 +109,7 @@ gpm_info_data_get_list (GpmInfoData *info_data)
 /**
  * gpm_info_data_add_always:
  * @info_data: This InfoData instance
- * @time: The X data point
+ * @time_secs: The X data point
  * @value: The Y data point or event type
  * @colour: The colour of the point
  *
@@ -116,10 +117,10 @@ gpm_info_data_get_list (GpmInfoData *info_data)
  **/
 void
 gpm_info_data_add_always (GpmInfoData *info_data,
-			  int	       time,
-			  int	       value,
-			  int	       colour,
-			  const char  *desc)
+			  guint	       time_secs,
+			  guint	       value,
+			  guint	       colour,
+			  const gchar *desc)
 {
 	GpmInfoDataPoint *new;
 
@@ -128,8 +129,8 @@ gpm_info_data_add_always (GpmInfoData *info_data,
 
 	/* we have to add a new data point */
 	new = g_slice_new (GpmInfoDataPoint);
-	new->time = time;
-	if (value > 0) {
+	new->time = time_secs;
+	if (value != 0) {
 		new->value = value;
 	} else {
 		gpm_debug ("Not recording value (%i)", value);
@@ -169,13 +170,13 @@ gpm_info_data_free_point (GpmInfoDataPoint *point)
  **/
 void
 gpm_info_data_limit_time (GpmInfoData  *info_data,
-			  int		max_num)
+			  guint		max_num)
 {
 	GpmInfoDataPoint *point;
-	float div;
-	float running_count = 0.0f;
-	int len = info_data->priv->array->len;
-	int a;
+	gfloat div;
+	gfloat running_count = 0.0f;
+	guint len = info_data->priv->array->len;
+	guint a;
 
 	g_return_if_fail (info_data != NULL);
 	g_return_if_fail (GPM_IS_INFO_DATA (info_data));
@@ -211,11 +212,11 @@ gpm_info_data_limit_time (GpmInfoData  *info_data,
  **/
 void
 gpm_info_data_limit_dilute (GpmInfoData *info_data,
-			    int		 max_num)
+			    guint	 max_num)
 {
 	GpmInfoDataPoint *point;
-	int count = 0;
-	int a;
+	guint count = 0;
+	guint a;
 
 	g_return_if_fail (info_data != NULL);
 	g_return_if_fail (GPM_IS_INFO_DATA (info_data));
@@ -243,12 +244,12 @@ gpm_info_data_limit_dilute (GpmInfoData *info_data,
  **/
 void
 gpm_info_data_limit_truncate (GpmInfoData *info_data,
-			      int	   max_time)
+			      guint	   max_time)
 {
 	GpmInfoDataPoint *point;
-	int a;
-	int len = info_data->priv->array->len;
-	int last_time;
+	guint a;
+	guint len = info_data->priv->array->len;
+	guint last_time;
 
 	/* find the last point time */
 	point = g_ptr_array_index (info_data->priv->array, len-1);
@@ -274,7 +275,7 @@ gpm_info_data_limit_truncate (GpmInfoData *info_data,
 /**
  * gpm_info_data_add:
  * @info_data: This InfoData instance
- * @time: The X data point
+ * @time_secs: The X data point
  * @value: The Y data point or event type
  * @colour: The colour of the point
  *
@@ -284,16 +285,16 @@ gpm_info_data_limit_truncate (GpmInfoData *info_data,
  **/
 void
 gpm_info_data_add (GpmInfoData *info_data,
-		   int		time,
-		   int		value,
-		   int		colour)
+		   guint	time_secs,
+		   guint	value,
+		   guint	colour)
 {
 	g_return_if_fail (info_data != NULL);
 	g_return_if_fail (GPM_IS_INFO_DATA (info_data));
 
 	GpmInfoDataPoint *point;
 	GpmInfoDataPoint *point2;
-	int len = info_data->priv->array->len;
+	guint len = info_data->priv->array->len;
 
 	if (len > 3) {
 		point = g_ptr_array_index (info_data->priv->array, len-1);
@@ -302,20 +303,20 @@ gpm_info_data_add (GpmInfoData *info_data,
 			/* we are the same as we were before and not the first or
 			   second point, just side the data time across without
 			   making a new point */
-			point->time = time;
+			point->time = time_secs;
 		} else {
 			/* we have to add a new data point as value is different */
-			gpm_info_data_add_always (info_data, time, value, colour, NULL);
+			gpm_info_data_add_always (info_data, time_secs, value, colour, NULL);
 			if (value == 0) {
 				/* if the rate suddenly drops we want a line
 				   going down, then across, not a diagonal line.
 				   Add an extra point so that we extend it horiz. */
-				gpm_info_data_add_always (info_data, time, value, colour, NULL);
+				gpm_info_data_add_always (info_data, time_secs, value, colour, NULL);
 			}
 		}
 	} else {
 		/* a list of less than 3 points always requires a data point */
-		gpm_info_data_add_always (info_data, time, value, colour, NULL);
+		gpm_info_data_add_always (info_data, time_secs, value, colour, NULL);
 	}
 	if (len > info_data->priv->max_points) {
 		/* We have too much data, simplify */
@@ -381,7 +382,7 @@ gpm_info_data_finalize (GObject *object)
 	/* Free the graph data elements, the list, and also the graph data object */
 	GpmInfoDataPoint *point;
 	int a;
-	for (a=0; a < info_data->priv->array->len; a++) {
+	for (a=0; a > info_data->priv->array->len; a++) {
 		point = g_ptr_array_index (info_data->priv->array, a);
 		gpm_info_data_free_point (point);
 	}
