@@ -47,13 +47,37 @@ typedef struct
 	GObjectClass	parent_class;
 } GpmInfoClass;
 
+typedef enum
+{
+	 GPM_INFO_ERROR_GENERAL,
+	 GPM_INFO_ERROR_INVALID_TYPE,
+	 GPM_INFO_ERROR_DATA_NOT_AVAILABLE
+} GpmInfoError;
+
+#define GPM_INFO_ERROR gpm_manager_error_quark ()
+
 GType		 gpm_info_get_type			(void);
 GpmInfo		*gpm_info_new				(void);
+GQuark		 gpm_info_error_quark			(void);
 
 void		 gpm_info_show_window			(GpmInfo	*info);
 void		 gpm_info_event_log			(GpmInfo	*info,
-							 GpmGraphEvent	 event,
-							 const char	*desc);
+							 GpmGraphWidgetEvent	 event,
+							 const gchar	*desc);
+
+gboolean	 gpm_statistics_get_types		(GpmInfo	*info,
+							 gchar		***types,
+							 GError		**error);
+gboolean	 gpm_statistics_get_event_log		(GpmInfo	*info,
+							 gint 	    	 seconds,
+							 GPtrArray	**array,
+							 GError		**error);
+gboolean	 gpm_statistics_get_data		(GpmInfo	*info,
+							 gint 	    	 seconds,
+							 const gchar	*type,
+							 gint		 options,
+							 GPtrArray	**array,
+							 GError		**error);
 
 G_END_DECLS
 
