@@ -30,26 +30,26 @@
 #include <string.h>
 #include "gpm-inhibit.h"
 #include "gpm-debug.h"
-#include "gpm-dbus-session-monitor.h"
+#include "gpm-dbus-monitor.h"
 
 static void     gpm_inhibit_class_init (GpmInhibitClass *klass);
 static void     gpm_inhibit_init       (GpmInhibit      *inhibit);
-static void     gpm_inhibit_finalize   (GObject      *object);
+static void     gpm_inhibit_finalize   (GObject		*object);
 
 #define GPM_INHIBIT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_INHIBIT, GpmInhibitPrivate))
 
 typedef struct
 {
-	char		*application;
-	char		*reason;
-	char		*connection;
+	gchar		*application;
+	gchar		*reason;
+	gchar		*connection;
 	guint32		 cookie;
 } GpmInhibitData;
 
 struct GpmInhibitPrivate
 {
 	GSList		*list;
-	GpmDbusSessionMonitor	*dbus_session;
+	GpmDbusMonitor	*dbus_monitor;
 };
 
 G_DEFINE_TYPE (GpmInhibit, gpm_inhibit, G_TYPE_OBJECT)
@@ -132,11 +132,13 @@ gpm_inhibit_generate_cookie (GpmInhibit *inhibit)
  * Return value: a new random cookie.
  **/
 guint32
-gpm_inhibit_add (GpmInhibit *inhibit,
-		 const char *connection,
-		 const char *application,
-		 const char *reason)
+gpm_inhibit_add (GpmInhibit  *inhibit,
+		 const gchar *connection,
+		 const gchar *application,
+		 const gchar *reason)
 {
+	GpmInhibitData *data = g_new (GpmInhibitData, 1);
+
 	/* handle where the application does not add required data */
 	if (connection == NULL ||
 	    application == NULL ||
@@ -147,7 +149,6 @@ gpm_inhibit_add (GpmInhibit *inhibit,
 	}
 
 	/* seems okay, add to list */
-	GpmInhibitData *data = g_new (GpmInhibitData, 1);
 	data->cookie = gpm_inhibit_generate_cookie (inhibit);
 	data->application = g_strdup (application);
 	data->connection = g_strdup (connection);
@@ -180,9 +181,9 @@ gpm_inhibit_free_data_object (GpmInhibitData *data)
  * Removes a cookie and associated data from the GpmInhibitData struct.
  **/
 void
-gpm_inhibit_remove (GpmInhibit *inhibit,
-		    const char *connection,
-		    guint32	cookie)
+gpm_inhibit_remove (GpmInhibit  *inhibit,
+		    const gchar *connection,
+		    guint32	 cookie)
 {
 	GpmInhibitData *data;
 
@@ -210,8 +211,8 @@ gpm_inhibit_remove (GpmInhibit *inhibit,
  * unregister it.
  **/
 static void
-gpm_inhibit_remove_dbus (GpmInhibit *inhibit,
-			 const char *connection)
+gpm_inhibit_remove_dbus (GpmInhibit  *inhibit,
+			 const gchar *connection)
 {
 	int a;
 	GpmInhibitData *data;
@@ -230,21 +231,21 @@ gpm_inhibit_remove_dbus (GpmInhibit *inhibit,
 }
 
 /**
- * dbus_name_owner_changed_session_cb:
+ * dbus_noc_session_cb:
  * @power: The power class instance
  * @name: The DBUS name, e.g. hal.freedesktop.org
  * @prev: The previous name, e.g. :0.13
  * @new: The new name, e.g. :0.14
  * @inhibit: This inhibit class instance
  *
- * The name-owner-changed session DBUS callback.
+ * The noc session DBUS callback.
  **/
 static void
-dbus_name_owner_changed_session_cb (GpmDbusSessionMonitor *dbus_monitor,
-				    const char	   *name,
-				    const char     *prev,
-				    const char     *new,
-				    GpmInhibit	   *inhibit)
+dbus_noc_session_cb (GpmDbusMonitor *dbus_monitor,
+		     const gchar    *name,
+		     const gchar    *prev,
+		     const gchar    *new,
+		     GpmInhibit	    *inhibit)
 {
 	if (strlen (new) == 0) {
 		gpm_inhibit_remove_dbus (inhibit, name);
@@ -279,11 +280,11 @@ gpm_inhibit_check (GpmInhibit *inhibit)
  *
  **/
 void
-gpm_inhibit_get_message (GpmInhibit *inhibit,
-			 GString    *message,
-			 const char *action)
+gpm_inhibit_get_message (GpmInhibit  *inhibit,
+			 GString     *message,
+			 const gchar *action)
 {
-	int a;
+	guint a;
 	GpmInhibitData *data;
 
 	if (g_slist_length (inhibit->priv->list) == 1) {
@@ -318,9 +319,9 @@ gpm_inhibit_init (GpmInhibit *inhibit)
 {
 	inhibit->priv = GPM_INHIBIT_GET_PRIVATE (inhibit);
 	inhibit->priv->list = NULL;
-	inhibit->priv->dbus_session = gpm_dbus_session_monitor_new ();
-	g_signal_connect (inhibit->priv->dbus_session, "name-owner-changed",
-			  G_CALLBACK (dbus_name_owner_changed_session_cb), inhibit);
+	inhibit->priv->dbus_monitor = gpm_dbus_monitor_new ();
+	g_signal_connect (inhibit->priv->dbus_monitor, "noc-session",
+			  G_CALLBACK (dbus_noc_session_cb), inhibit);
 }
 
 /** finalise the object */
@@ -328,6 +329,9 @@ static void
 gpm_inhibit_finalize (GObject *object)
 {
 	GpmInhibit *inhibit;
+	guint a;
+	GpmInhibitData *data;
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GPM_IS_INHIBIT (object));
 
@@ -335,15 +339,13 @@ gpm_inhibit_finalize (GObject *object)
 	inhibit->priv = GPM_INHIBIT_GET_PRIVATE (inhibit);
 
 	/* remove items in list and free */
-	int a;
-	GpmInhibitData *data;
 	for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
 		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
 		gpm_inhibit_free_data_object (data);
 	}
 	g_slist_free (inhibit->priv->list);
 
-	g_object_unref (inhibit->priv->dbus_session);
+	g_object_unref (inhibit->priv->dbus_monitor);
 	G_OBJECT_CLASS (gpm_inhibit_parent_class)->finalize (object);
 }
 
