@@ -34,6 +34,7 @@
 #include "gpm-common.h"
 #include "gpm-hal.h"
 #include "gpm-hal-monitor.h"
+#include "gpm-hal-power.h"
 
 #include "gpm-power.h"
 #include "gpm-marshal.h"
@@ -51,11 +52,12 @@ static void     gpm_power_finalize   (GObject       *object);
 struct GpmPowerPrivate
 {
 	gboolean		 on_ac;
-	int			 exp_ave_factor;
+	gint			 exp_ave_factor;
 	GHashTable		*battery_kind_cache;
 	GHashTable		*battery_device_cache;
 	GpmHal			*hal;
 	GpmHalMonitor		*hal_monitor;
+	GpmHalPower		*hal_power;
 };
 
 enum {
@@ -143,8 +145,8 @@ battery_device_cache_entry_update_all (GpmPower *power, GpmPowerDevice *entry)
 {
 	gboolean exists;
 	GpmPowerStatus *status = &entry->battery_status;
-	char *udi = entry->udi;
-	char *battery_kind_str;
+	gchar *udi = entry->udi;
+	gchar *battery_kind_str;
 
 	/* invalidate last rate */
 	entry->charge_rate_previous = 0;
@@ -279,19 +281,19 @@ battery_device_cache_entry_update_all (GpmPower *power, GpmPowerDevice *entry)
  * does not change drastically between updates.
  **/
 static int
-gpm_power_exp_aver (int previous, int new, int factor_pc)
+gpm_power_exp_aver (gint previous, gint new, gint factor_pc)
 {
-	int result = 0;
-	float factor = 0;
-	float factor_inv = 1;
+	gint result = 0;
+	gfloat factor = 0;
+	gfloat factor_inv = 1;
 	if (previous == 0 || factor_pc == 0) {
 		/* startup, or re-initialization - we have no data */
 		gpm_debug ("Telling rate with no ave factor (okay once)");
 		result = new;
 	} else {
-		factor = (float) factor_pc / 100.0f;
+		factor = (gfloat) factor_pc / 100.0f;
 		factor_inv = 1.0f - factor;
-		result = (int) ((factor_inv * (float) new) + (factor * (float) previous));
+		result = (gint) ((factor_inv * (gfloat) new) + (factor * (gfloat) previous));
 	}
 	gpm_debug ("factor = %f, previous = %i, new=%i, result = %i",
 		   factor, previous, new, result);
@@ -307,12 +309,12 @@ gpm_power_exp_aver (int previous, int new, int factor_pc)
  * Updates the device object with the new information given to us from HAL.
  **/
 static void
-battery_device_cache_entry_update_key (GpmPower		       *power,
+battery_device_cache_entry_update_key (GpmPower	      *power,
 				       GpmPowerDevice *entry,
-				       const char	       *key)
+				       const gchar    *key)
 {
 	GpmPowerStatus *status = &entry->battery_status;
-	char *udi = entry->udi;
+	gchar *udi = entry->udi;
 
 	if (key == NULL) {
 		return;
@@ -369,7 +371,8 @@ battery_device_cache_entry_update_key (GpmPower		       *power,
  * Return value: The new device cache entry.
  **/
 static GpmPowerDevice *
-battery_device_cache_entry_new_from_udi (GpmPower *power, const char *udi)
+battery_device_cache_entry_new_from_udi (GpmPower *power,
+					 const gchar *udi)
 {
 	GpmPowerDevice *entry;
 
@@ -447,10 +450,10 @@ gpm_power_kind_to_localised_string (GpmPowerKind battery_kind)
  * @battery_kind: The type of battery, e.g. GPM_POWER_KIND_PRIMARY
  * Return value: The raw battery kind, e.g. primary. Do not free this string.
  **/
-const char *
+const gchar *
 gpm_power_kind_to_string (GpmPowerKind battery_kind)
 {
-	const char *str;
+	const gchar *str;
 
 	if (battery_kind == GPM_POWER_KIND_PRIMARY) {
  		str = "primary";
@@ -528,8 +531,8 @@ battery_kind_cache_debug_print_all (GpmPower *power)
  * Return value: The entry if found, or NULL if missing.
  **/
 static GpmPowerDevice *
-battery_device_cache_find (GpmPower   *power,
-			   const char *udi)
+battery_device_cache_find (GpmPower    *power,
+			   const gchar *udi)
 {
 	GpmPowerDevice *entry;
 
@@ -557,8 +560,8 @@ battery_device_cache_find (GpmPower   *power,
  * Return value: The entry if found, or NULL if missing.
  **/
 static BatteryKindCacheEntry *
-battery_kind_cache_find (GpmPower		*power,
-			 GpmPowerKind	 battery_kind)
+battery_kind_cache_find (GpmPower     *power,
+			 GpmPowerKind  battery_kind)
 {
 	BatteryKindCacheEntry *entry;
 
@@ -579,8 +582,8 @@ battery_kind_cache_find (GpmPower		*power,
  * Return value: the number of devices of a specific kind.
  **/
 gint
-gpm_power_get_num_devices_of_kind (GpmPower		*power,
-				   GpmPowerKind	 battery_kind)
+gpm_power_get_num_devices_of_kind (GpmPower    *power,
+				   GpmPowerKind	battery_kind)
 {
 	BatteryKindCacheEntry *entry;
 	if (power->priv->battery_kind_cache == NULL) {
@@ -596,11 +599,11 @@ gpm_power_get_num_devices_of_kind (GpmPower		*power,
 GpmPowerDevice *
 gpm_power_get_battery_device_entry (GpmPower	 *power,
 				    GpmPowerKind  battery_kind,
-				    gint	  device_num)
+				    guint	  device_num)
 {
-	const char		*udi;
+	const gchar *udi;
 	GpmPowerDevice *device;
-	BatteryKindCacheEntry   *entry;
+	BatteryKindCacheEntry *entry;
 
 	if (! power->priv->battery_kind_cache) {
 		return NULL;
@@ -614,7 +617,7 @@ gpm_power_get_battery_device_entry (GpmPower	 *power,
 	}
 
 	/* get the udi of the battery we are interested in */
-	udi = (const char *) g_slist_nth_data (entry->devices, device_num);
+	udi = (const gchar *) g_slist_nth_data (entry->devices, device_num);
 
 	/* find the udi in the device cache */
 	device = battery_device_cache_find (power, udi);
@@ -671,7 +674,6 @@ GString *
 gpm_power_status_for_device_more (GpmPowerDevice *device)
 {
 	GString		*details;
-	const char	*suffix;
 	GpmPowerStatus	*status;
 
 	status = &device->battery_status;
@@ -681,14 +683,23 @@ gpm_power_status_for_device_more (GpmPowerDevice *device)
 		g_string_append_printf (details, _("<b>Vendor:</b> %s\n"), device->vendor);
 	}
 	if (device->technology) {
-		const char *technology;
+		const gchar *technology;
+/* we can remove these when we depend on HAL 0.5.8 */
+#if 1
 		if (strcasecmp (device->technology, "li-ion") == 0 ||
-		    strcasecmp (device->technology, "lithium-ion") == 0 ||
 		    strcasecmp (device->technology, "lion") == 0) {
 			technology = _("Lithium ion");
-		} else if (strcasecmp (device->technology, "pbac") == 0 ||
-			   strcasecmp (device->technology, "lead-acid") == 0) {
+		} else if (strcasecmp (device->technology, "pbac") == 0) {
 			technology = _("Lead acid");
+#endif
+		} else if (strcmp (device->technology, "lithium-ion") == 0) {
+			technology = _("Lithium ion");
+		} else if (strcasecmp (device->technology, "lead-acid") == 0) {
+			technology = _("Lead acid");
+		} else if (strcasecmp (device->technology, "lithium-polymer") == 0) {
+			technology = _("Lithium polymer");
+		} else if (strcasecmp (device->technology, "nickel-metal-hydride") == 0) {
+			technology = _("Nickel metal hydride");
 		} else {
 			gpm_warning ("Battery type %s not translated, please report!",
 				     device->technology);
@@ -703,10 +714,10 @@ gpm_power_status_for_device_more (GpmPowerDevice *device)
 		g_string_append_printf (details, _("<b>Model:</b> %s\n"), device->model);
 	}
 	if (status->remaining_time > 0) {
-		char *time;
-		time = gpm_get_timestring (status->remaining_time);
-		g_string_append_printf (details, _("<b>Remaining time:</b> %s\n"), time);
-		g_free (time);
+		char *time_str;
+		time_str = gpm_get_timestring (status->remaining_time);
+		g_string_append_printf (details, _("<b>Remaining time:</b> %s\n"), time_str);
+		g_free (time_str);
 	}
 	if (status->capacity > 0) {
 		const char *condition;
@@ -723,38 +734,36 @@ gpm_power_status_for_device_more (GpmPowerDevice *device)
 					status->capacity, condition);
 	}
 	if (device->unit == GPM_POWER_UNIT_MWH) {
-		suffix = "W";
 		if (status->current_charge > 0) {
-			g_string_append_printf (details, _("<b>Current charge:</b> %.1f %s\n"),
-						status->current_charge / 1000.0f, suffix);
+			g_string_append_printf (details, _("<b>Current charge:</b> %.1f W\n"),
+						status->current_charge / 1000.0f);
 		}
 		if (status->last_full_charge > 0 &&
 		    status->design_charge != status->last_full_charge) {
-			g_string_append_printf (details, _("<b>Last full charge:</b> %.1f %s\n"),
-						status->last_full_charge / 1000.0f, suffix);
+			g_string_append_printf (details, _("<b>Last full charge:</b> %.1f W\n"),
+						status->last_full_charge / 1000.0f);
 		}
 		if (status->design_charge > 0) {
-			g_string_append_printf (details, _("<b>Design charge:</b> %.1f %s\n"),
-						status->design_charge / 1000.0f, suffix);
+			g_string_append_printf (details, _("<b>Design charge:</b> %.1f W\n"),
+						status->design_charge / 1000.0f);
 		}
 		if (status->charge_rate_raw > 0) {
-			g_string_append_printf (details, _("<b>Charge rate (raw):</b> %.1f %sh\n"),
-						status->charge_rate_raw / 1000.0f, suffix);
+			g_string_append_printf (details, _("<b>Charge rate (raw):</b> %.1f Wh\n"),
+						status->charge_rate_raw / 1000.0f);
 		}
 		if (status->charge_rate_smoothed > 0) {
-			g_string_append_printf (details, _("<b>Charge rate (smoothed):</b> %.1f %sh\n"),
-						status->charge_rate_smoothed / 1000.0f, suffix);
+			g_string_append_printf (details, _("<b>Charge rate (smoothed):</b> %.1f Wh\n"),
+						status->charge_rate_smoothed / 1000.0f);
 		}
 	}
 	if (device->unit == GPM_POWER_UNIT_CSR) {
-		suffix = "/ 7";
 		if (status->current_charge > 0) {
-			g_string_append_printf (details, _("<b>Current charge:</b> %.1f %s\n"),
-						(float) status->current_charge, suffix);
+			g_string_append_printf (details, _("<b>Current charge:</b> %i/7\n"),
+						status->current_charge);
 		}
 		if (status->design_charge > 0) {
-			g_string_append_printf (details, _("<b>Design charge:</b> %.1f %s\n"),
-						(float) status->design_charge, suffix);
+			g_string_append_printf (details, _("<b>Design charge:</b> %i/7\n"),
+						status->design_charge);
 		}
 	}
 	/* remove the last \n */
@@ -776,7 +785,7 @@ gpm_power_status_for_device_more (GpmPowerDevice *device)
  *
  * Return value: The character string for the filename suffix.
  **/
-static char *
+static const gchar *
 gpm_power_get_index_from_percent (gint percent)
 {
 	if (percent < 10) {
@@ -803,12 +812,12 @@ gpm_power_get_index_from_percent (gint percent)
  *
  * Return value: The complete filename, must free using g_free.
  **/
-static char *
+static gchar *
 gpm_power_get_icon_for_all (GpmPowerStatus *device_status,
-			    const char     *prefix)
+			    const gchar    *prefix)
 {
 	char *filename = NULL;
-	char *index = NULL;
+	const char *index_str = NULL;
 
 	if (! device_status->is_present) {
 		/* battery missing */
@@ -818,12 +827,12 @@ gpm_power_get_icon_for_all (GpmPowerStatus *device_status,
 		filename = g_strdup_printf ("gpm-%s-charged", prefix);
 
 	} else if (device_status->is_charging) {
-		index = gpm_power_get_index_from_percent (device_status->percentage_charge);
-		filename = g_strdup_printf ("gpm-%s-%s-charging", prefix, index);
+		index_str = gpm_power_get_index_from_percent (device_status->percentage_charge);
+		filename = g_strdup_printf ("gpm-%s-%s-charging", prefix, index_str);
 
 	} else if (device_status->is_discharging) {
-		index = gpm_power_get_index_from_percent (device_status->percentage_charge);
-		filename = g_strdup_printf ("gpm-%s-%s", prefix, index);
+		index_str = gpm_power_get_index_from_percent (device_status->percentage_charge);
+		filename = g_strdup_printf ("gpm-%s-%s", prefix, index_str);
 
 	} else {
 		/* We have a broken battery, not sure what to display here */
@@ -842,23 +851,23 @@ gpm_power_get_icon_for_all (GpmPowerStatus *device_status,
  *
  * Return value: The complete filename, must free using g_free.
  **/
-static char *
+static gchar *
 gpm_power_get_icon_for_csr (GpmPowerStatus *device_status,
-			    const char     *prefix)
+			    const gchar    *prefix)
 {
-	char *filename;
-	char *index;
+	gchar *filename;
+	const gchar *index_str;
 
 	if (device_status->current_charge < 2) {
-		index = "000";
+		index_str = "000";
 	} else if (device_status->current_charge < 4) {
-		index = "030";
+		index_str = "030";
 	} else if (device_status->current_charge < 6) {
-		index = "060";
+		index_str = "060";
 	} else {
-		index = "100";
+		index_str = "100";
 	}
-	filename = g_strdup_printf ("gpm-%s-%s", prefix, index);
+	filename = g_strdup_printf ("gpm-%s-%s", prefix, index_str);
 	return filename;
 }
 
@@ -871,12 +880,12 @@ gpm_power_get_icon_for_csr (GpmPowerStatus *device_status,
  *
  * Return value: The complete filename, must free using g_free.
  **/
-char *
+gchar *
 gpm_power_get_icon_from_status (GpmPowerStatus *device_status,
 				GpmPowerKind    kind)
 {
-	char	   *filename = NULL;
-	const char *prefix;
+	gchar *filename = NULL;
+	const gchar *prefix;
 
 	/* TODO: icons need to be renamed from -battery- to -primary- */
 	prefix = gpm_power_kind_to_string (kind);
@@ -911,8 +920,8 @@ battery_kind_cache_update (GpmPower		 *power,
 			   BatteryKindCacheEntry *entry)
 {
 	GSList *l;
-	int     num_present = 0;
-	int     num_discharging = 0;
+	guint num_present = 0;
+	guint num_discharging = 0;
 	GpmPowerStatus *type_status = &entry->battery_status;
 
 	/* clear old values */
@@ -920,11 +929,11 @@ battery_kind_cache_update (GpmPower		 *power,
 
 	/* iterate thru all the devices to handle multiple batteries */
 	for (l = entry->devices; l; l = l->next) {
-		const char	      *udi;
+		const gchar *udi;
 		GpmPowerDevice *device;
 		GpmPowerStatus	*device_status;
 
-		udi = (const char *)l->data;
+		udi = (const gchar *)l->data;
 
 		device = battery_device_cache_find (power, udi);
 		device_status = &device->battery_status;
@@ -966,14 +975,30 @@ battery_kind_cache_update (GpmPower		 *power,
 		type_status->is_charging = FALSE;
 	}
 
+	/* If the primary battery is neither charging nor discharging, and
+	 * the charge is low the battery is most likely broken.
+	 * In this case, we'll use the ac_adaptor to determine whether it's
+	 * charging or not. */
+	if (entry->battery_kind == GPM_POWER_KIND_PRIMARY &&
+	    (! type_status->is_charging) && (! type_status->is_discharging) &&
+	    type_status->percentage_charge > 0 &&
+	    type_status->percentage_charge < GPM_POWER_MIN_CHARGED_PERCENTAGE) {
+		gboolean on_ac;
+		on_ac = gpm_hal_power_is_on_ac (power->priv->hal_power);
+		gpm_debug ("Battery is neither charging nor discharging, "
+			   "using ac_adaptor value %i", on_ac);
+		type_status->is_charging = on_ac;
+		type_status->is_discharging = !(on_ac);
+	}
+
 	gpm_debug ("%i devices of type %s", num_present, gpm_power_kind_to_localised_string (entry->battery_kind));
 
 	/* Perform following calculations with floating point otherwise we might
 	 * get an with batteries which have a very small charge unit and consequently
 	 * a very high charge. Fixes bug #327471 */
 	if (type_status->is_present) {
-		int pc = 100 * ((float)type_status->current_charge /
-				(float)type_status->last_full_charge);
+		gint pc = 100 * ((gfloat)type_status->current_charge /
+				(gfloat)type_status->last_full_charge);
 		if (pc < 0) {
 			gpm_warning ("Corrected percentage charge (%i) and set to minimum", pc);
 			pc = 0;
@@ -983,22 +1008,6 @@ battery_kind_cache_update (GpmPower		 *power,
 		}
 		type_status->percentage_charge = pc;
 	}
-
-	/* If the primary battery is neither charging nor discharging, and
-	 * the charge is low the battery is most likely broken.
-	 * In this case, we'll use the ac_adaptor to determine whether it's
-	 * charging or not. */
-	if (entry->battery_kind == GPM_POWER_KIND_PRIMARY &&
-	    type_status->is_charging == FALSE &&
-	    type_status->is_discharging == FALSE &&
-	    type_status->percentage_charge > 0 &&
-	    type_status->percentage_charge < GPM_POWER_MIN_CHARGED_PERCENTAGE) {
-		gpm_debug ("Battery is neither charging nor discharging, "
-			   "using ac_adaptor value %i", power->priv->on_ac);
-		type_status->is_charging = power->priv->on_ac;
-		type_status->is_discharging = !(power->priv->on_ac);
-	}
-
 	/* We only do the "better" remaining time algorithm if the battery has rate,
 	   i.e not a UPS, which gives it's own battery.remaining_time but has no rate */
 	if (type_status->charge_rate_smoothed > 0) {
@@ -1028,7 +1037,7 @@ battery_kind_cache_update (GpmPower		 *power,
  * battery_kind_update_cache_iter:
  **/
 static void
-battery_kind_update_cache_iter (const char	      *key,
+battery_kind_update_cache_iter (const gchar	      *key,
 				BatteryKindCacheEntry *entry,
 				GpmPower	      *power)
 {
@@ -1059,7 +1068,7 @@ battery_kind_cache_update_all (GpmPower *power)
  * @entry: A device cache instance
  **/
 static void
-battery_device_cache_add_device (GpmPower		 *power,
+battery_device_cache_add_device (GpmPower       *power,
 				 GpmPowerDevice *entry)
 {
 	g_hash_table_insert (power->priv->battery_device_cache,
@@ -1073,7 +1082,7 @@ battery_device_cache_add_device (GpmPower		 *power,
  * @entry: A device cache instance
  **/
 static void
-battery_device_cache_remove_device (GpmPower		    *power,
+battery_device_cache_remove_device (GpmPower	   *power,
 				    GpmPowerDevice *entry)
 {
 	g_hash_table_remove (power->priv->battery_device_cache,
@@ -1100,7 +1109,7 @@ battery_device_cache_remove_device (GpmPower		    *power,
  * Adds a device entry to the correct cache entry.
  **/
 static void
-battery_kind_cache_add_device (GpmPower			*power,
+battery_kind_cache_add_device (GpmPower		*power,
 			       GpmPowerDevice	*device_entry)
 {
 	BatteryKindCacheEntry *type_entry;
@@ -1132,7 +1141,7 @@ battery_kind_cache_add_device (GpmPower			*power,
  * Removes a device entry from a cache entry.
  **/
 static void
-battery_kind_cache_remove_device (GpmPower		  *power,
+battery_kind_cache_remove_device (GpmPower	 *power,
 				  GpmPowerDevice *entry)
 {
 	BatteryKindCacheEntry *type_entry;
@@ -1164,14 +1173,14 @@ battery_kind_cache_remove_device (GpmPower		  *power,
  * mainly.
  **/
 static void
-power_get_summary_for_battery_kind (GpmPower		*power,
+power_get_summary_for_battery_kind (GpmPower	 *power,
 			    	    GpmPowerKind  battery_kind,
-			    	    GString		*summary)
+			    	    GString	 *summary)
 {
 	BatteryKindCacheEntry *entry;
 	GpmPowerStatus *status;
-	const char	      *type_desc = NULL;
-	char		      *timestring;
+	const gchar *type_desc = NULL;
+	gchar *timestring;
 
 	entry = battery_kind_cache_find (power, battery_kind);
 
@@ -1202,30 +1211,32 @@ power_get_summary_for_battery_kind (GpmPower		*power,
 	/* We always display "Laptop Battery 16 minutes remaining" as we need
 	   to clarify what device we are refering to. For details see :
 	   http://bugzilla.gnome.org/show_bug.cgi?id=329027 */
-	g_string_append_printf (summary, "%s ", type_desc);
 
 	if (gpm_power_battery_is_charged (status)) {
 
-			g_string_append (summary, _("fully charged"));
+			g_string_append_printf (summary, _("%s fully charged (%i%%)\n"),
+						type_desc, status->percentage_charge);
 
 	} else if (status->is_discharging) {
 
 		if (status->remaining_time > 60) {
-			g_string_append_printf (summary, _("%s remaining"),
-						timestring);
+			g_string_append_printf (summary, _("%s %s remaining (%i%%)\n"),
+						type_desc, timestring, status->percentage_charge);
 		} else {
 			/* don't display "Unknown remaining" */
-			g_string_append (summary, _("discharging"));
+			g_string_append_printf (summary, _("%s discharging (%i%%)\n"),
+						type_desc, status->percentage_charge);
 		}
 
 	} else if (status->is_charging || power->priv->on_ac) {
 
 		if (status->remaining_time > 60) {
-			g_string_append_printf (summary, _("%s until charged"),
-						timestring);
+			g_string_append_printf (summary, _("%s %s until charged (%i%%)\n"),
+						type_desc, timestring, status->percentage_charge);
 		} else {
 			/* don't display "Unknown remaining" */
-			g_string_append (summary, _("charging"));
+			g_string_append_printf (summary, _("%s charging (%i%%)\n"),
+						type_desc, status->percentage_charge);
 		}
 
 	} else {
@@ -1233,9 +1244,6 @@ power_get_summary_for_battery_kind (GpmPower		*power,
 			     "discharging and the batteries are also not charged");
 	}
 
-	/* append percentage to all devices */
-	g_string_append_printf (summary, " (%i%%)\n", status->percentage_charge);
-
 	g_free (timestring);
 }
 
@@ -1248,7 +1256,7 @@ power_get_summary_for_battery_kind (GpmPower		*power,
  **/
 gboolean
 gpm_power_get_status_summary (GpmPower *power,
-			      char    **string,
+			      gchar   **string,
 			      GError  **error)
 {
 	GString *summary = NULL;
@@ -1301,9 +1309,9 @@ gpm_power_get_status_summary (GpmPower *power,
  * Return value: if the device was found in the cache.
  **/
 gboolean
-gpm_power_get_battery_status (GpmPower			 *power,
-			      GpmPowerKind	  battery_kind,
-			      GpmPowerStatus      *battery_status)
+gpm_power_get_battery_status (GpmPower       *power,
+			      GpmPowerKind    battery_kind,
+			      GpmPowerStatus *battery_status)
 {
 	BatteryKindCacheEntry *entry;
 
@@ -1500,8 +1508,8 @@ hal_on_ac_changed_cb (GpmHalMonitor *monitor,
  * correct kind cache depending on type.
  **/
 static gboolean
-add_battery (GpmPower   *power,
-	     const char *udi)
+add_battery (GpmPower    *power,
+	     const gchar *udi)
 {
 	GpmPowerDevice *entry;
 	GpmPowerStatus *status;
@@ -1542,8 +1550,8 @@ add_battery (GpmPower   *power,
  * Return value: if a battery was removed.
  **/
 static gboolean
-remove_battery (GpmPower   *power,
-		const char *udi)
+remove_battery (GpmPower    *power,
+		const gchar *udi)
 {
 	GpmPowerDevice *entry;
 
@@ -1573,7 +1581,7 @@ remove_battery (GpmPower   *power,
  **/
 static void
 hal_battery_added_cb (GpmHalMonitor *monitor,
-		      const char    *udi,
+		      const gchar   *udi,
 		      GpmPower      *power)
 {
 	gpm_debug ("Battery Added: %s", udi);
@@ -1591,7 +1599,7 @@ hal_battery_added_cb (GpmHalMonitor *monitor,
  **/
 static void
 hal_battery_removed_cb (GpmHalMonitor *monitor,
-			const char    *udi,
+			const gchar   *udi,
 			GpmPower      *power)
 {
 	gpm_debug ("Battery Removed: %s", udi);
@@ -1615,8 +1623,8 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
  **/
 static void
 hal_battery_property_modified_cb (GpmHalMonitor *monitor,
-				  const char    *udi,
-				  const char    *key,
+				  const gchar   *udi,
+				  const gchar   *key,
 				  gboolean	 finally,
 				  GpmPower      *power)
 {
@@ -1665,7 +1673,7 @@ hal_battery_property_modified_cb (GpmHalMonitor *monitor,
  **/
 static void
 hal_button_pressed_cb (GpmHalMonitor *monitor,
-		       const char    *type,
+		       const gchar   *type,
 		       gboolean       state,
 		       GpmPower      *power)
 {
@@ -1775,7 +1783,7 @@ gpm_power_update_all (GpmPower *power)
  **/
 static void
 hal_daemon_start_cb (GpmHal     *hal,
-		    GpmPower   *power)
+		     GpmPower   *power)
 {
 	gpm_hash_new_kind_cache (power);
 	gpm_hash_new_device_cache (power);
@@ -1792,8 +1800,8 @@ hal_daemon_start_cb (GpmHal     *hal,
  * and we segfault in various places.
  **/
 static void
-hal_daemon_stop_cb (GpmHal     *hal,
-		    GpmPower   *power)
+hal_daemon_stop_cb (GpmHal   *hal,
+		    GpmPower *power)
 {
 	gpm_hash_free_kind_cache (power);
 	gpm_hash_free_device_cache (power);
@@ -1830,13 +1838,15 @@ gpm_power_init (GpmPower *power)
 	g_signal_connect (power->priv->hal_monitor, "battery-removed",
 			  G_CALLBACK (hal_battery_removed_cb), power);
 
+	power->priv->hal_power = gpm_hal_power_new ();
+
 	power->priv->battery_kind_cache = NULL;
 	power->priv->battery_device_cache = NULL;
 
 	gpm_hash_new_kind_cache (power);
 	gpm_hash_new_device_cache (power);
 
-	on_ac = gpm_hal_is_on_ac (power->priv->hal);
+	on_ac = gpm_hal_power_is_on_ac (power->priv->hal_power);
 	gpm_power_set_on_ac (power, on_ac, NULL);
 
 	power->priv->exp_ave_factor = gconf_client_get_int (client,
@@ -1863,8 +1873,9 @@ gpm_power_finalize (GObject *object)
 	gpm_hash_free_kind_cache (power);
 	gpm_hash_free_device_cache (power);
 
-	g_object_unref (power->priv->hal_monitor);
 	g_object_unref (power->priv->hal);
+	g_object_unref (power->priv->hal_monitor);
+	g_object_unref (power->priv->hal_power);
 
 	G_OBJECT_CLASS (gpm_power_parent_class)->finalize (object);
 }
