@@ -33,10 +33,10 @@
 #include <math.h>
 #include <string.h>
 
-#include "gpm-screensaver.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
 #include "gpm-hal.h"
+#include "gpm-hal-cpufreq.h"
 #include "gpm-prefs-core.h"
 #include "gpm-debug.h"
 #include "gpm-stock-icons.h"
@@ -56,9 +56,13 @@ struct GpmPrefsPrivate
 	gboolean		 has_lcd;
 	gboolean		 has_ups;
 	gboolean		 has_button_lid;
+	gboolean		 has_cpufreq;
 	gboolean		 can_suspend;
 	gboolean		 can_hibernate;
 	GpmScreensaver		*screensaver;
+	GpmHalCpuFreq		*cpufreq;
+	GpmHalCpuFreqEnum	 cpufreq_types;
+	GpmHal			*hal;
 };
 
 enum {
@@ -88,6 +92,14 @@ static GConfEnumStringPair icon_policy_enum_map [] = {
 #define ACTION_BLANK_TEXT		_("Blank screen")
 #define ACTION_NOTHING_TEXT		_("Do nothing")
 
+/* The text that should appear in the processor combo box */
+#define CPUFREQ_ONDEMAND_TEXT		_("Based on processor load")
+#define CPUFREQ_CONSERVATIVE_TEXT	_("Automatic power saving")
+#define CPUFREQ_POWERSAVE_TEXT		_("Maximum power saving")
+#define CPUFREQ_USERSPACE_TEXT		_("Custom")
+#define CPUFREQ_PERFORMANCE_TEXT	_("Maximum speed")
+#define CPUFREQ_NOTHING_TEXT 		_("Do nothing")
+
 #define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
 #define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
 #define	GPM_DBUS_INTERFACE		"org.gnome.PowerManager"
@@ -126,17 +138,31 @@ gpm_prefs_class_init (GpmPrefsClass *klass)
 			      G_TYPE_NONE, 0);
 }
 
+/**
+ * gpm_prefs_activate_window:
+ * @prefs: This prefs class instance
+ *
+ * Activates (shows) the window.
+ **/
+void
+gpm_prefs_activate_window (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, "window_preferences");
+	gtk_window_present (GTK_WINDOW (widget));
+}
+
 /**
  * gpm_dbus_method_bool:
  * @method: The g-p-m DBUS method name, e.g. "AllowedSuspend"
  **/
 static gboolean
-gpm_dbus_method_bool (const char *method)
+gpm_dbus_method_bool (const gchar *method)
 {
 	DBusGConnection *connection;
-	DBusGProxy      *proxy;
-	GError	   *error;
-	gboolean	  value;
+	DBusGProxy *proxy;
+	GError *error;
+	gboolean value;
 
 	value = FALSE;
 	error = NULL;
@@ -194,8 +220,8 @@ static void
 gpm_prefs_icon_radio_cb (GtkWidget *widget,
 			 GpmPrefs  *prefs)
 {
-	const char  *str;
-	int	     policy;
+	const gchar *str;
+	gint policy;
 
 	policy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "policy"));
 	str = gconf_enum_to_string (icon_policy_enum_map, policy);
@@ -206,12 +232,12 @@ gpm_prefs_icon_radio_cb (GtkWidget *widget,
 }
 
 /**
- * gpm_prefs_format_brightness_cb:
+ * gpm_prefs_format_percentage_cb:
  * @scale: The GtkScale object
  * @value: The value in %.
  **/
-static char *
-gpm_prefs_format_brightness_cb (GtkScale *scale,
+static gchar *
+gpm_prefs_format_percentage_cb (GtkScale *scale,
 				gdouble   value)
 {
 	return g_strdup_printf ("%.0f%%", value);
@@ -223,12 +249,12 @@ gpm_prefs_format_brightness_cb (GtkScale *scale,
  * @value: The value in minutes.
  * @prefs: This prefs class instance
  **/
-static char *
+static gchar *
 gpm_prefs_format_time_cb (GtkScale *scale,
 			  gdouble   value,
 			  GpmPrefs *prefs)
 {
-	char *str;
+	gchar *str;
 	if ((gint) value == NEVER_TIME_ON_SLIDER) {
 		str = g_strdup (_("Never"));
 	} else {
@@ -281,9 +307,9 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget *
-gpm_prefs_setup_sleep_slider (GpmPrefs  *prefs,
-			      char	*widget_name,
-			      char	*gpm_pref_key)
+gpm_prefs_setup_sleep_slider (GpmPrefs    *prefs,
+			      const gchar *widget_name,
+			      const gchar *gpm_pref_key)
 {
 	GtkWidget *widget;
 	gint value;
@@ -331,7 +357,7 @@ gpm_prefs_brightness_slider_changed_cb (GtkRange *range,
 					GpmPrefs *prefs)
 {
 	gdouble value;
-	char *gpm_pref_key;
+	gchar *gpm_pref_key;
 
 	value = gtk_range_get_value (range);
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "gconf_key");
@@ -348,9 +374,9 @@ gpm_prefs_brightness_slider_changed_cb (GtkRange *range,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget *
-gpm_prefs_setup_brightness_slider (GpmPrefs *prefs,
-				   char     *widget_name,
-				   char     *gpm_pref_key)
+gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
+				   const gchar *widget_name,
+				   const gchar *gpm_pref_key)
 {
 	GladeXML    *xml = prefs->priv->glade_xml;
 	GtkWidget *widget;
@@ -360,7 +386,7 @@ gpm_prefs_setup_brightness_slider (GpmPrefs *prefs,
 	widget = glade_xml_get_widget (xml, widget_name);
 
 	g_signal_connect (G_OBJECT (widget), "format-value",
-			  G_CALLBACK (gpm_prefs_format_brightness_cb), NULL);
+			  G_CALLBACK (gpm_prefs_format_percentage_cb), NULL);
 
 	value = gconf_client_get_int (prefs->priv->gconf_client, gpm_pref_key, NULL);
 	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
@@ -387,9 +413,9 @@ static void
 gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 				   GpmPrefs  *prefs)
 {
-	char *value;
-	char *action;
-	char *gpm_pref_key;
+	gchar *value;
+	const gchar *action;
+	gchar *gpm_pref_key;
 
 	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 
@@ -423,15 +449,15 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
  * @actions: The actions to associate in an array.
  **/
 static void
-gpm_prefs_setup_action_combo (GpmPrefs    *prefs,
-			      char	  *widget_name,
-			      char	  *gpm_pref_key,
-			      const char **actions)
+gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
+			      const gchar  *widget_name,
+			      const gchar  *gpm_pref_key,
+			      const gchar **actions)
 {
 	GladeXML    *xml = prefs->priv->glade_xml;
-	char *value;
-	int i = 0;
-	int n_added = 0;
+	gchar *value;
+	gint i = 0;
+	gint n_added = 0;
 	gboolean is_writable;
 	GtkWidget *widget;
 
@@ -444,7 +470,7 @@ gpm_prefs_setup_action_combo (GpmPrefs    *prefs,
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
-	if (! value) {
+	if (value == NULL) {
 		gpm_warning ("invalid schema, please re-install");
 		value = g_strdup ("nothing");
 	}
@@ -505,10 +531,28 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
 			    GpmPrefs  *prefs)
 {
 	gboolean checked;
-	char *gpm_pref_key;
+	gchar *gpm_pref_key;
+	GtkWidget *twidget;
+	const gchar *widget_name;
 
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
+	widget_name = gtk_widget_get_name (widget);
+	if (widget_name && strcmp (widget_name, "checkbutton_display_state_change") == 0) {
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"label_display_ac_brightness");
+		gtk_widget_set_sensitive (twidget, checked);
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"hscale_display_ac_brightness");
+		gtk_widget_set_sensitive (twidget, checked);
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"label_display_battery_brightness");
+		gtk_widget_set_sensitive (twidget, checked);
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"hscale_display_battery_brightness");
+		gtk_widget_set_sensitive (twidget, checked);
+	}
+
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "gconf_key");
 	gpm_debug ("Changing %s to %i", gpm_pref_key, checked);
 	gconf_client_set_bool (prefs->priv->gconf_client, gpm_pref_key, checked, NULL);
@@ -521,9 +565,9 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget*
-gpm_prefs_setup_checkbox (GpmPrefs  *prefs,
-			  char	    *widget_name,
-			  char	    *gpm_pref_key)
+gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
+			  const gchar *widget_name,
+			  const gchar *gpm_pref_key)
 {
 
 	GladeXML    *xml = prefs->priv->glade_xml;
@@ -541,197 +585,529 @@ gpm_prefs_setup_checkbox (GpmPrefs  *prefs,
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_checkbox_lock_cb), prefs);
 
+	/* manually do the callback in case we hide elements in the cb */
+	gpm_prefs_checkbox_lock_cb (widget, prefs);
+
 	return widget;
 }
 
 /**
- * setup_battery_sliders:
+ * gpm_prefs_close_cb:
+ * @widget: The GtkWidget object
  * @prefs: This prefs class instance
  **/
 static void
-setup_battery_sliders (GpmPrefs *prefs)
+gpm_prefs_close_cb (GtkWidget	*widget,
+		    GpmPrefs	*prefs)
 {
-	GtkWidget   *vbox_battery_brightness;
+	gpm_debug ("emitting action-close");
+	g_signal_emit (prefs, signals [ACTION_CLOSE], 0);
+}
 
-	/* Sleep time on batteries */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_computer",
-				      GPM_PREF_BATTERY_SLEEP_COMPUTER);
+/**
+ * gpm_prefs_delete_event_cb:
+ * @widget: The GtkWidget object
+ * @event: The event type, unused.
+ * @prefs: This prefs class instance
+ **/
+static gboolean
+gpm_prefs_delete_event_cb (GtkWidget *widget,
+			  GdkEvent   *event,
+			  GpmPrefs   *prefs)
+{
+	gpm_prefs_close_cb (widget, prefs);
+	return FALSE;
+}
 
-	/* Sleep time for display when on batteries */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_display",
-				      GPM_PREF_BATTERY_SLEEP_DISPLAY);
+/**
+ * set_idle_hscale_stops:
+ * @prefs: This prefs class instance
+ * @widget_name: The widget name
+ *
+ * Here we make sure that the start of the hscale is set to the
+ * gnome-screensaver idle time to avoid confusion.
+ **/
+static void
+set_idle_hscale_stops (GpmPrefs    *prefs,
+		       const gchar *widget_name,
+		       gint         gs_idle_time)
+{
+	GtkWidget *widget;
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
+	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
+		gpm_warning ("gnome-screensaver timeout is really big. "
+			     "Not sure what to do");
+		return;
+	}
+	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
+}
 
-	/* Display brightness when on batteries */
-	gpm_prefs_setup_brightness_slider (prefs, "hscale_battery_brightness",
-					   GPM_PREF_BATTERY_BRIGHTNESS);
+/**
+ * gs_delay_changed_cb:
+ * @key: The gconf key
+ * @prefs: This prefs class instance
+ **/
+static void
+gs_delay_changed_cb (GpmScreensaver *screensaver,
+		     gint	     delay,
+		     GpmPrefs	    *prefs)
+{
+	/* update the start and stop points on the hscales */
+	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
+	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
+	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
+	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
+}
 
-	if (! prefs->priv->has_lcd) {
-		vbox_battery_brightness = glade_xml_get_widget (prefs->priv->glade_xml,
-								"vbox_battery_brightness");
-		gtk_widget_hide_all (vbox_battery_brightness);
+/**
+ * gconf_key_changed_cb:
+ *
+ * We might have to do things when the gconf keys change; do them here.
+ **/
+static void
+gconf_key_changed_cb (GConfClient *client,
+		      guint	   cnxn_id,
+		      GConfEntry  *entry,
+		      gpointer	   user_data)
+{
+	GpmPrefs *prefs = GPM_PREFS (user_data);
+	gboolean  enabled;
+
+	gpm_debug ("Key changed %s", entry->key);
+
+	if (gconf_entry_get_value (entry) == NULL) {
+		return;
+	}
+
+	if (strcmp (entry->key, GPM_PREF_AC_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  		 GPM_PREF_AC_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
+
+	} else if (strcmp (entry->key, GPM_PREF_UPS_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  		 GPM_PREF_UPS_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
+
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_LOWPOWER) == 0) {
+		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
+				  		 GPM_PREF_BATTERY_LOWPOWER, NULL);
+		gpm_debug ("need to enable checkbox");
 	}
 }
 
 /**
- * setup_ac_sliders:
- * @prefs: This prefs class instance
+ * gpm_prefs_processor_slider_changed_cb:
+ * @range: The GtkRange object
+ * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static void
-setup_ac_sliders (GpmPrefs *prefs)
+gpm_prefs_processor_slider_changed_cb (GtkRange *range,
+				       GpmPrefs *prefs)
 {
-	GladeXML    *xml = prefs->priv->glade_xml;
-	GtkWidget   *vbox_ac_brightness;
+	gint value;
+	gchar *gpm_pref_key;
 
-	/* Sleep time on AC */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_computer",
-				      GPM_PREF_AC_SLEEP_COMPUTER);
-
-	/* Sleep time for display on AC */
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_display",
-				      GPM_PREF_AC_SLEEP_DISPLAY);
-
-	/* Display brightness when on AC */
-	gpm_prefs_setup_brightness_slider (prefs, "hscale_ac_brightness",
-					   GPM_PREF_AC_BRIGHTNESS);
-
-	if (! prefs->priv->has_lcd) {
-		vbox_ac_brightness = glade_xml_get_widget (xml, "vbox_ac_brightness");
-		gtk_widget_hide_all (vbox_ac_brightness);
-	}
+	value = (gint) gtk_range_get_value (range);
+	gpm_pref_key = (gchar *) g_object_get_data (G_OBJECT (range), "gconf_key");
+	gpm_debug ("Changing %s to %i", gpm_pref_key, value);
+	gconf_client_set_int (prefs->priv->gconf_client, gpm_pref_key, value, NULL);
 }
 
 /**
- * setup_sleep_type:
+ * gpm_prefs_setup_processor_slider:
  * @prefs: This prefs class instance
+ * @widget_name: The GtkWidget name
+ * @gpm_pref_key: The GConf key for this preference setting.
+ **/
+static GtkWidget *
+gpm_prefs_setup_processor_slider (GpmPrefs   *prefs,
+				  const gchar *widget_name,
+				  const gchar *gpm_pref_key)
+{
+	GtkWidget *widget;
+	gint value;
+	gboolean is_writable;
+
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
+	g_signal_connect (G_OBJECT (widget), "format-value",
+			  G_CALLBACK (gpm_prefs_format_percentage_cb), prefs);
+
+	value = gconf_client_get_int (prefs->priv->gconf_client, gpm_pref_key, NULL);
+
+	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
+						    gpm_pref_key, NULL);
+
+	gtk_widget_set_sensitive (widget, is_writable);
+
+	gtk_range_set_value (GTK_RANGE (widget), value);
+
+	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+
+	g_signal_connect (G_OBJECT (widget), "value-changed",
+			  G_CALLBACK (gpm_prefs_processor_slider_changed_cb),
+			  prefs);
+
+	return widget;
+}
+
+/**
+ * gpm_prefs_processor_combo_changed_cb:
+ * @widget: The GtkWidget object
+ * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static void
-setup_sleep_type (GpmPrefs *prefs)
+gpm_prefs_processor_combo_changed_cb (GtkWidget *widget,
+				      GpmPrefs  *prefs)
 {
-	GtkWidget    *checkbutton_dim_idle;
-	const char   *sleep_type_actions[] = {ACTION_NOTHING,
-					      ACTION_SUSPEND,
-					      ACTION_HIBERNATE,
-					      NULL};
+	gchar *value;
+	const gchar *policy;
+	gchar *gpm_pref_key;
+	gboolean show_custom = FALSE;
+	GtkWidget *twidget;
+
+	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
+	if (value == NULL) {
+		g_warning ("active text failed");
+		return;
+	}
+	if (strcmp (value, CPUFREQ_ONDEMAND_TEXT) == 0) {
+		policy = CODE_CPUFREQ_ONDEMAND;
+	} else if (strcmp (value, CPUFREQ_CONSERVATIVE_TEXT) == 0) {
+		policy = CODE_CPUFREQ_CONSERVATIVE;
+	} else if (strcmp (value, CPUFREQ_POWERSAVE_TEXT) == 0) {
+		policy = CODE_CPUFREQ_POWERSAVE;
+	} else if (strcmp (value, CPUFREQ_USERSPACE_TEXT) == 0) {
+		policy = CODE_CPUFREQ_USERSPACE;
+		show_custom = TRUE;
+	} else if (strcmp (value, CPUFREQ_PERFORMANCE_TEXT) == 0) {
+		policy = CODE_CPUFREQ_PERFORMANCE;
+	} else if (strcmp (value, CPUFREQ_NOTHING_TEXT) == 0) {
+		policy = CODE_CPUFREQ_NOTHING;
+	} else {
+		g_assert (FALSE);
+	}
+
+	/* show other options */
+	if (strcmp (gtk_widget_get_name (widget), "combobox_processor_ac_profile") == 0) {
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"label_processor_ac_custom");
+		gtk_widget_set_sensitive (twidget, show_custom);
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"hscale_processor_ac_custom");
+		gtk_widget_set_sensitive (twidget, show_custom);
+	} else {
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"label_processor_battery_custom");
+		gtk_widget_set_sensitive (twidget, show_custom);
+		twidget = glade_xml_get_widget (prefs->priv->glade_xml,
+						"hscale_processor_battery_custom");
+		gtk_widget_set_sensitive (twidget, show_custom);
+	}
+
+	g_free (value);
+	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "gconf_key");
+	gpm_debug ("Changing %s to %s", gpm_pref_key, policy);
+	gconf_client_set_string (prefs->priv->gconf_client, gpm_pref_key, policy, NULL);
+}
+
+/**
+ * gpm_prefs_setup_action_combo:
+ * @prefs: This prefs class instance
+ * @widget_name: The GtkWidget name
+ * @gpm_pref_key: The GConf key for this preference setting.
+ * @actions: The actions to associate in an array.
+ **/
+static void
+gpm_prefs_setup_processor_combo (GpmPrefs         *prefs,
+				 const gchar      *widget_name,
+				 const gchar      *gpm_pref_key,
+				 GpmHalCpuFreqEnum cpufreq_types)
+{
+	gchar *value;
+	guint n_added = 0;
+	gboolean has_option = FALSE;
+	gboolean is_writable;
+	GtkWidget *widget;
+	GpmHalCpuFreqEnum cpufreq_type;
+
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
+	value = gconf_client_get_string (prefs->priv->gconf_client,
+					 gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
+						    gpm_pref_key, NULL);
+
+	gtk_widget_set_sensitive (widget, is_writable);
+
+	if (value == NULL) {
+		gpm_warning ("invalid schema, please re-install");
+		value = g_strdup ("nothing");
+	}
+
+	g_object_set_data (G_OBJECT (widget), "gconf_key", (gpointer) gpm_pref_key);
+	g_signal_connect (G_OBJECT (widget), "changed",
+			  G_CALLBACK (gpm_prefs_processor_combo_changed_cb),
+			  prefs);
+
+	cpufreq_type = gpm_hal_cpufreq_string_to_enum (value);
+
+	if (cpufreq_types & GPM_CPUFREQ_ONDEMAND) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_ONDEMAND_TEXT);
+		if (cpufreq_type == GPM_CPUFREQ_ONDEMAND) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+	if (cpufreq_types & GPM_CPUFREQ_CONSERVATIVE) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_CONSERVATIVE_TEXT);
+		if (cpufreq_type == GPM_CPUFREQ_CONSERVATIVE) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+	if (cpufreq_types & GPM_CPUFREQ_POWERSAVE) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_POWERSAVE_TEXT);
+		if (cpufreq_type == GPM_CPUFREQ_POWERSAVE) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+	if (cpufreq_types & GPM_CPUFREQ_USERSPACE) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_USERSPACE_TEXT);
+		if (cpufreq_type == GPM_CPUFREQ_USERSPACE) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+	if (cpufreq_types & GPM_CPUFREQ_PERFORMANCE) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+					   CPUFREQ_PERFORMANCE_TEXT);
+		if (cpufreq_type == GPM_CPUFREQ_PERFORMANCE) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+			has_option = TRUE;
+		}
+		n_added++;
+	}
+
+	/* always add nothing as a valid option */
+	gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
+				   CPUFREQ_NOTHING_TEXT);
+	if (has_option == FALSE || cpufreq_type == GPM_CPUFREQ_NOTHING) {
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added);
+	}
+	g_free (value);
+}
+
+/** setup the sleep page */
+static void
+prefs_setup_sleep (GpmPrefs *prefs)
+{
+	GtkWidget *widget;
+	gint delay;
+
+	const gchar *sleep_type_actions[] = {ACTION_NOTHING,
+					     ACTION_SUSPEND,
+					     ACTION_HIBERNATE,
+					     NULL};
 
 	/* Sleep Type Combo Box */
-
-	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_type",
+	gpm_prefs_setup_action_combo (prefs, "combobox_sleep_general_type",
 				      GPM_PREF_SLEEP_TYPE,
 				      sleep_type_actions);
+	/* Sleep time until we sleep */
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ac_inactive",
+				      GPM_PREF_AC_SLEEP_COMPUTER);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_battery_inactive",
+				      GPM_PREF_BATTERY_SLEEP_COMPUTER);
+//FIXME
+//	gpm_prefs_setup_sleep_slider (prefs, "hscale_sleep_ups_inactive",
+//				      GPM_PREF_UPS_SLEEP_COMPUTER);
 
-	/* set up the "do we dim screen on idle checkbox */
-	checkbutton_dim_idle = gpm_prefs_setup_checkbox (prefs, "checkbutton_dim_idle",
-				  			 GPM_PREF_IDLE_DIM_SCREEN);
+	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	set_idle_hscale_stops (prefs, "hscale_sleep_battery_inactive", delay);
+	set_idle_hscale_stops (prefs, "hscale_sleep_ac_inactive", delay);
 
-	if (! prefs->priv->has_lcd) {
-		gtk_widget_hide_all (checkbutton_dim_idle);
+	if (prefs->priv->has_batteries == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_sleep_battery");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_ups == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_sleep_ups");
+		gtk_widget_hide_all (widget);
 	}
 }
 
-/**
- * setup_ac_actions:
- * @prefs: This prefs class instance
- **/
+/** setup the display page */
 static void
-setup_ac_actions (GpmPrefs *prefs)
+prefs_setup_display (GpmPrefs *prefs)
 {
-	GtkWidget    *checkbox_powersave;
-	GtkWidget    *vbox_ac_actions;
-	const char   *button_lid_actions[] = {ACTION_NOTHING,
+	GtkWidget *widget;
+	gint delay;
+
+	/* Sleep time for display to blank */
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_ac_sleep",
+				      GPM_PREF_AC_SLEEP_DISPLAY);
+	gpm_prefs_setup_sleep_slider (prefs, "hscale_display_battery_sleep",
+				      GPM_PREF_BATTERY_SLEEP_DISPLAY);
+	/* Display brightness */
+	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_ac_brightness",
+					   GPM_PREF_AC_BRIGHTNESS);
+	gpm_prefs_setup_brightness_slider (prefs, "hscale_display_battery_brightness",
+					   GPM_PREF_BATTERY_BRIGHTNESS);
+
+	/* set up the general checkboxes */
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_dim",
+				  GPM_PREF_DISPLAY_IDLE_DIM);
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_state_change",
+				  GPM_PREF_DISPLAY_STATE_CHANGE);
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_display_ambient",
+				  GPM_PREF_DISPLAY_STATE_CHANGE);
+	/* for now, hide */
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_ambient");
+	gtk_widget_hide_all (widget);
+
+	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
+	set_idle_hscale_stops (prefs, "hscale_display_battery_sleep", delay);
+	set_idle_hscale_stops (prefs, "hscale_display_ac_sleep", delay);
+
+	if (prefs->priv->has_lcd == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hscale_display_ac_brightness");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_display_ac_brightness");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hscale_display_battery_brightness");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_display_battery_brightness");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_ac_dim");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "checkbutton_display_battery_dim");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_batteries == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_display_battery");
+		gtk_widget_hide_all (widget);
+	}
+}
+
+/** setup the processor page */
+static void
+prefs_setup_processor (GpmPrefs *prefs)
+{
+	GtkWidget    *widget;
+	GtkWidget    *notebook;
+	gint	      page;
+
+	/* remove if we have no options to set */
+	if (prefs->priv->has_cpufreq == FALSE) {
+		notebook = glade_xml_get_widget (prefs->priv->glade_xml, "notebook_preferences");
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_processor");
+		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
+		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+		return;
+	}
+
+	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_battery_custom",
+					  GPM_PREF_BATTERY_CPUFREQ_VALUE);
+	gpm_prefs_setup_processor_slider (prefs, "hscale_processor_ac_custom",
+					  GPM_PREF_AC_CPUFREQ_VALUE);
+
+	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_ac_profile",
+					 GPM_PREF_AC_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+	gpm_prefs_setup_processor_combo (prefs, "combobox_processor_battery_profile",
+					 GPM_PREF_BATTERY_CPUFREQ_POLICY, prefs->priv->cpufreq_types);
+}
+
+/** setup the actions page */
+static void
+prefs_setup_actions (GpmPrefs *prefs)
+{
+	GtkWidget    *widget;
+	const gchar  *button_lid_actions[] = {ACTION_NOTHING,
 					      ACTION_BLANK,
 					      ACTION_SUSPEND,
 					      ACTION_HIBERNATE,
 					      NULL};
-
-	gpm_prefs_setup_action_combo (prefs, "combobox_ac_lid_close",
-				      GPM_PREF_AC_BUTTON_LID,
-				      button_lid_actions);
-
-	checkbox_powersave = gpm_prefs_setup_checkbox (prefs,
-						       "checkbutton_ac_powersave",
-						       GPM_PREF_AC_LOWPOWER);
-	/* hide elements that do not apply */
-	if (! prefs->priv->has_batteries) {
-		gtk_widget_hide_all (checkbox_powersave);
-		if (!prefs->priv->has_button_lid) {
-			/* hide the whole vbox if we don't have a lid */
-			vbox_ac_actions = glade_xml_get_widget (prefs->priv->glade_xml,
-								"vbox_ac_actions");
-			gtk_widget_hide_all (vbox_ac_actions);
-		}
-	}
-}
-
-/**
- * setup_battery_actions:
- * @prefs: This prefs class instance
- **/
-static void
-setup_battery_actions (GpmPrefs *prefs)
-{
-	GtkWidget    *label_button_lid;
-	GtkWidget    *combo_button_lid;
-	const char   *button_lid_actions[] = {ACTION_NOTHING,
-					      ACTION_BLANK,
-					      ACTION_SUSPEND,
-					      ACTION_HIBERNATE,
-					      NULL};
-	const char   *battery_critical_actions[] = {ACTION_NOTHING,
+	const gchar  *battery_critical_actions[] = {ACTION_NOTHING,
 						    ACTION_SUSPEND,
 						    ACTION_HIBERNATE,
 						    ACTION_SHUTDOWN,
 						    NULL};
-
-	/* Button Lid Combo Box */
-	gpm_prefs_setup_action_combo (prefs, "combobox_battery_lid_close",
-				      GPM_PREF_BATTERY_BUTTON_LID,
-				      button_lid_actions);
-
-	if (! prefs->priv->has_button_lid) {
-		label_button_lid = glade_xml_get_widget (prefs->priv->glade_xml,
-							 "label_battery_button_lid");
-		combo_button_lid = glade_xml_get_widget (prefs->priv->glade_xml,
-							 "combobox_battery_lid_close");
-		gtk_widget_hide_all (label_button_lid);
-		gtk_widget_hide_all (combo_button_lid);
-	}
-
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_powersave",
-	  			  GPM_PREF_BATTERY_LOWPOWER);
-
-	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
-				      GPM_PREF_BATTERY_CRITICAL,
-				      battery_critical_actions);
-}
-
-/**
- * setup_ups_actions:
- * @prefs: This prefs class instance
- **/
-static void
-setup_ups_actions (GpmPrefs *prefs)
-{
-	const char   *battery_ups_actions[] = {ACTION_NOTHING,
+	const gchar  *battery_ups_actions[] = {ACTION_NOTHING,
 					       ACTION_HIBERNATE,
 					       ACTION_SHUTDOWN,
 					       NULL};
-	gpm_prefs_setup_checkbox (prefs, "checkbutton_ups_powersave",
-	  			  GPM_PREF_UPS_LOWPOWER);
+	const gchar  *power_button_actions[] = {ACTION_INTERACTIVE,
+						ACTION_SUSPEND,
+						ACTION_HIBERNATE,
+						ACTION_SHUTDOWN,
+						NULL};
+	/* Power button action */
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_general_power",
+				      GPM_PREF_BUTTON_POWER,
+				      power_button_actions);
 
-	gpm_prefs_setup_action_combo (prefs, "combobox_ups_critical",
+	/* Lid close actions */
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ac_lid",
+				      GPM_PREF_AC_BUTTON_LID,
+				      button_lid_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_lid",
+				      GPM_PREF_BATTERY_BUTTON_LID,
+				      button_lid_actions);
+
+	/* set up the LowPowerMode checkbox */
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_actions_battery_low_power",
+	  			  GPM_PREF_BATTERY_LOWPOWER);
+
+	/* battery critical actions */
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_battery_critical",
+				      GPM_PREF_BATTERY_CRITICAL,
+				      battery_critical_actions);
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_critical",
 				      GPM_PREF_UPS_CRITICAL,
 				      battery_ups_actions);
+
+	gpm_prefs_setup_action_combo (prefs, "combobox_actions_ups_low",
+				      GPM_PREF_UPS_LOW,
+				      battery_ups_actions);
+
+	if (prefs->priv->has_button_lid == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "combobox_actions_ac_lid");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "combobox_actions_battery_lid");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_actions_ac_lid");
+		gtk_widget_hide_all (widget);
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "label_actions_battery_lid");
+		gtk_widget_hide_all (widget);
+		/* remove whole box as there is nothing in it */
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_ac");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_ups == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_ups");
+		gtk_widget_hide_all (widget);
+	}
+	if (prefs->priv->has_batteries == FALSE) {
+		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_actions_battery");
+		gtk_widget_hide_all (widget);
+	}
 }
 
-/**
- * setup_icon_policy:
- * @prefs: This prefs class instance
- **/
+/** setup the notification page */
 static void
-setup_icon_policy (GpmPrefs *prefs)
+prefs_setup_notification (GpmPrefs *prefs)
 {
-	char	    *icon_policy_str;
-	int	     icon_policy;
+	gchar	    *icon_policy_str;
+	gint	     icon_policy;
 	GtkWidget   *radiobutton_icon_always;
 	GtkWidget   *radiobutton_icon_present;
 	GtkWidget   *radiobutton_icon_charge;
@@ -746,19 +1122,18 @@ setup_icon_policy (GpmPrefs *prefs)
 	g_free (icon_policy_str);
 
 	radiobutton_icon_always = glade_xml_get_widget (prefs->priv->glade_xml,
-							"radiobutton_icon_always");
+							"radiobutton_notification_always");
 	radiobutton_icon_present = glade_xml_get_widget (prefs->priv->glade_xml,
-							"radiobutton_icon_present");
+							"radiobutton_notification_present");
 	radiobutton_icon_charge = glade_xml_get_widget (prefs->priv->glade_xml,
-							"radiobutton_icon_charge");
+							"radiobutton_notification_charge");
 	radiobutton_icon_critical = glade_xml_get_widget (prefs->priv->glade_xml,
-							"radiobutton_icon_critical");
+							"radiobutton_notification_critical");
 	radiobutton_icon_never = glade_xml_get_widget (prefs->priv->glade_xml,
-							"radiobutton_icon_never");
+							"radiobutton_notification_never");
 
 	is_writable = gconf_client_key_is_writable (prefs->priv->gconf_client,
 						    GPM_PREF_ICON_POLICY, NULL);
-
 	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_present, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
@@ -800,118 +1175,17 @@ setup_icon_policy (GpmPrefs *prefs)
 	g_signal_connect (radiobutton_icon_never, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
 
-	if (! prefs->priv->has_batteries) {
+	/* set up the sound checkbox */
+	gpm_prefs_setup_checkbox (prefs, "checkbutton_notification_sound",
+	  			  GPM_PREF_ENABLE_BEEPING);
+
+	if (prefs->priv->has_batteries == FALSE) {
 		/* Hide battery radio options if we have no batteries */
 		gtk_widget_hide_all (radiobutton_icon_charge);
 		gtk_widget_hide_all (radiobutton_icon_critical);
 	}
 }
 
-/**
- * gpm_prefs_close_cb:
- * @widget: The GtkWidget object
- * @prefs: This prefs class instance
- **/
-static void
-gpm_prefs_close_cb (GtkWidget	*widget,
-		    GpmPrefs	*prefs)
-{
-	gpm_debug ("emitting action-close");
-	g_signal_emit (prefs, signals [ACTION_CLOSE], 0);
-}
-
-/**
- * gpm_prefs_delete_event_cb:
- * @widget: The GtkWidget object
- * @event: The event type, unused.
- * @prefs: This prefs class instance
- **/
-static gboolean
-gpm_prefs_delete_event_cb (GtkWidget	*widget,
-			  GdkEvent	*event,
-			  GpmPrefs	*prefs)
-{
-	gpm_prefs_close_cb (widget, prefs);
-	return FALSE;
-}
-
-/**
- * set_idle_hscale_stops:
- * @prefs: This prefs class instance
- * @widget_name: The widget name
- *
- * Here we make sure that the start of the hscale is set to the
- * gnome-screensaver idle time to avoid confusion.
- **/
-static void
-set_idle_hscale_stops (GpmPrefs   *prefs,
-		       const char *widget_name,
-		       int	   gs_idle_time)
-{
-	GtkWidget *widget;
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
-	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
-		gpm_warning ("gnome-screensaver timeout is really big. "
-			     "Not sure what to do");
-		return;
-	}
-	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
-}
-
-/**
- * gs_delay_changed_cb:
- * @key: The gconf key
- * @prefs: This prefs class instance
- **/
-static void
-gs_delay_changed_cb (GpmScreensaver *screensaver,
-		    int		     delay,
-		    GpmPrefs	    *prefs)
-{
-	/* update the start and stop points on the hscales */
-	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
-}
-
-/**
- * gconf_key_changed_cb:
- *
- * We might have to do things when the gconf keys change; do them here.
- **/
-static void
-gconf_key_changed_cb (GConfClient *client,
-		      guint	   cnxn_id,
-		      GConfEntry  *entry,
-		      gpointer	   user_data)
-{
-	GpmPrefs *prefs = GPM_PREFS (user_data);
-	gboolean  enabled;
-
-	gpm_debug ("Key changed %s", entry->key);
-
-	if (gconf_entry_get_value (entry) == NULL) {
-		return;
-	}
-
-	if (strcmp (entry->key, GPM_PREF_AC_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  	         GPM_PREF_AC_LOWPOWER, NULL);
-		gpm_debug ("need to enable checkbox");
-
-	} else if (strcmp (entry->key, GPM_PREF_UPS_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  	         GPM_PREF_UPS_LOWPOWER, NULL);
-		gpm_debug ("need to enable checkbox");
-
-	} else if (strcmp (entry->key, GPM_PREF_BATTERY_LOWPOWER) == 0) {
-		enabled = gconf_client_get_bool (prefs->priv->gconf_client,
-				  	         GPM_PREF_BATTERY_LOWPOWER, NULL);
-		gpm_debug ("need to enable checkbox");
-	}
-}
-
 /**
  * gpm_prefs_init:
  * @prefs: This prefs class instance
@@ -921,19 +1195,12 @@ gpm_prefs_init (GpmPrefs *prefs)
 {
 	GtkWidget    *main_window;
 	GtkWidget    *widget;
-	GtkWidget    *notebook;
-	int           delay;
-	int           page;
-
-	GpmHal       *hal = gpm_hal_new ();
-        const char   *power_button_actions[] = {ACTION_INTERACTIVE,
-                                                ACTION_SUSPEND,
-                                                ACTION_HIBERNATE,
-                                                ACTION_SHUTDOWN,
-                                                NULL};
 
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
+	prefs->priv->hal = gpm_hal_new ();
+	prefs->priv->cpufreq = gpm_hal_cpufreq_new ();
+
 	prefs->priv->screensaver = gpm_screensaver_new ();
 	g_signal_connect (prefs->priv->screensaver, "gs-delay-changed",
 			  G_CALLBACK (gs_delay_changed_cb), prefs);
@@ -947,21 +1214,33 @@ gpm_prefs_init (GpmPrefs *prefs)
 				 NULL,
 				 NULL);
 
-	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability (hal, "laptop_panel") > 0;
-	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value (hal, "battery",
+	prefs->priv->has_lcd = gpm_hal_num_devices_of_capability (prefs->priv->hal, "laptop_panel") > 0;
+	prefs->priv->has_batteries = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
 							"battery.type",
 							"primary") > 0;
-	prefs->priv->has_ups = gpm_hal_num_devices_of_capability_with_value (hal, "battery",
+	prefs->priv->has_ups = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "battery",
 							"battery.type",
 							"ups") > 0;
-	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value (hal, "button",
+	prefs->priv->has_button_lid = gpm_hal_num_devices_of_capability_with_value (prefs->priv->hal, "button",
 							"button.type",
 							"lid") > 0;
-	g_object_unref (hal);
 
 	prefs->priv->can_suspend = gpm_dbus_method_bool ("AllowedSuspend");
 	prefs->priv->can_hibernate = gpm_dbus_method_bool ("AllowedHibernate");
 
+	/* only enable cpufreq stuff if we have the hardware */
+	prefs->priv->has_cpufreq = gpm_hal_cpufreq_has_hardware (prefs->priv->cpufreq);
+	gboolean ret = FALSE;
+	if (prefs->priv->has_cpufreq) {
+		ret = gpm_hal_cpufreq_get_governors (prefs->priv->cpufreq,
+							   &prefs->priv->cpufreq_types);
+	}
+	/* method failed, can't assume we have cpu frequency scaling */
+	if (ret == FALSE) {
+		prefs->priv->cpufreq_types = GPM_CPUFREQ_NOTHING;
+		prefs->priv->has_cpufreq = FALSE;
+	}
+
 	prefs->priv->glade_xml = glade_xml_new (GPM_DATA "/gpm-prefs.glade", NULL, NULL);
 
 	main_window = glade_xml_get_widget (prefs->priv->glade_xml, "window_preferences");
@@ -983,37 +1262,20 @@ gpm_prefs_init (GpmPrefs *prefs)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_help_cb), prefs);
 
-	setup_icon_policy (prefs);
-	setup_ac_actions (prefs);
-	setup_ac_sliders (prefs);
-	setup_battery_actions (prefs);
-	setup_battery_sliders (prefs);
-	setup_ups_actions (prefs);
-	setup_sleep_type (prefs);
+	/* the 'Sleep' tab */
+	prefs_setup_sleep (prefs);
 
-	/* Power button action */
-	gpm_prefs_setup_action_combo (prefs, "combobox_power_button",
-				      GPM_PREF_BUTTON_POWER,
-				      power_button_actions);
+	/* the 'Display' tab */
+	prefs_setup_display (prefs);
 
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
+	/* the 'Processing Speed' tab */
+	prefs_setup_processor (prefs);
 
-	notebook = glade_xml_get_widget (prefs->priv->glade_xml, "gpm_notebook");
-	/* if no options then disable frame as it will be empty */
-	if (! prefs->priv->has_batteries) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_battery_power");
-		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
-		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
-	}
-	if (! prefs->priv->has_ups) {
-		widget = glade_xml_get_widget (prefs->priv->glade_xml, "vbox_ups_power");
-		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
-		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
-	}
+	/* the 'Actions' tab */
+	prefs_setup_actions (prefs);
+
+	/* the 'Notification' tab */
+	prefs_setup_notification (prefs);
 
 	gtk_widget_show (main_window);
 }
@@ -1034,6 +1296,8 @@ gpm_prefs_finalize (GObject *object)
 
 	g_object_unref (prefs->priv->gconf_client);
 	g_object_unref (prefs->priv->screensaver);
+	g_object_unref (prefs->priv->cpufreq);
+	g_object_unref (prefs->priv->hal);
 	/* FIXME: we should unref some stuff */
 
 	G_OBJECT_CLASS (gpm_prefs_parent_class)->finalize (object);
