@@ -36,6 +36,10 @@ G_BEGIN_DECLS
 #define GPM_PREF_BATTERY_DPMS_METHOD	GPM_PREF_DIR "/battery_dpms_sleep_method"
 #define GPM_PREF_BATTERY_BRIGHTNESS	GPM_PREF_DIR "/battery_brightness"
 
+#define GPM_PREF_DISPLAY_IDLE_DIM	GPM_PREF_DIR "/display_dim_on_idle"
+#define GPM_PREF_DISPLAY_STATE_CHANGE	GPM_PREF_DIR "/display_state_change"
+#define GPM_PREF_DISPLAY_AMBIENT	GPM_PREF_DIR "/display_ambient"
+
 #define GPM_PREF_SLEEP_TYPE		GPM_PREF_DIR "/action_sleep_type"
 #define GPM_PREF_BUTTON_SUSPEND		GPM_PREF_DIR "/action_button_suspend"
 #define GPM_PREF_BUTTON_HIBERNATE	GPM_PREF_DIR "/action_button_hibernate"
@@ -44,6 +48,7 @@ G_BEGIN_DECLS
 #define GPM_PREF_BATTERY_BUTTON_LID	GPM_PREF_DIR "/action_battery_button_lid"
 #define GPM_PREF_BATTERY_CRITICAL	GPM_PREF_DIR "/action_battery_critical"
 #define GPM_PREF_UPS_CRITICAL		GPM_PREF_DIR "/action_ups_critical"
+#define GPM_PREF_UPS_LOW		GPM_PREF_DIR "/action_ups_low"
 
 #define GPM_PREF_USE_TIME_POLICY	GPM_PREF_DIR "/use_time_for_policy"
 #define GPM_PREF_ICON_POLICY		GPM_PREF_DIR "/display_icon_policy"
@@ -54,6 +59,8 @@ G_BEGIN_DECLS
 #define GPM_PREF_CAN_SUSPEND		GPM_PREF_DIR "/can_suspend"
 #define GPM_PREF_CAN_HIBERNATE		GPM_PREF_DIR "/can_hibernate"
 
+#define GPM_PREF_NETWORKMANAGER_SLEEP	GPM_PREF_DIR "/networkmanager_sleep"
+
 #define GPM_PREF_LOCK_USE_SCREENSAVER	GPM_PREF_DIR "/lock_use_screensaver_settings"
 /* These are only effective if the system default is turned off. See bug #331164 */
 #define GPM_PREF_LOCK_ON_BLANK_SCREEN	GPM_PREF_DIR "/lock_on_blank_screen"
@@ -61,8 +68,6 @@ G_BEGIN_DECLS
 #define GPM_PREF_LOCK_ON_HIBERNATE	GPM_PREF_DIR "/lock_on_hibernate"
 
 #define GPM_PREF_IDLE_CHECK_CPU		GPM_PREF_DIR "/check_type_cpu"
-#define GPM_PREF_IDLE_CHECK_NET		GPM_PREF_DIR "/check_type_net"
-#define GPM_PREF_IDLE_DIM_SCREEN	GPM_PREF_DIR "/dim_on_idle"
 
 /* This allows us to ignore policy on resume for a bit */
 #define GPM_PREF_POLICY_TIMEOUT		GPM_PREF_DIR "/policy_suppression_timeout"
@@ -91,6 +96,14 @@ G_BEGIN_DECLS
 #define GPM_PREF_UPS_LOWPOWER		GPM_PREF_DIR "/use_lowpower_ups"
 #define GPM_PREF_BATTERY_LOWPOWER	GPM_PREF_DIR "/use_lowpower_battery"
 
+#define GPM_PREF_AC_CPUFREQ_POLICY	GPM_PREF_DIR "/cpufreq_ac_policy"
+#define GPM_PREF_AC_CPUFREQ_VALUE	GPM_PREF_DIR "/cpufreq_ac_performance"
+#define GPM_PREF_BATTERY_CPUFREQ_POLICY	GPM_PREF_DIR "/cpufreq_battery_policy"
+#define GPM_PREF_BATTERY_CPUFREQ_VALUE	GPM_PREF_DIR "/cpufreq_battery_performance"
+#define GPM_PREF_USE_NICE		GPM_PREF_DIR "/cpufreq_consider_nice"
+
+#define GPM_PREF_SESSION_REQUEST_SAVE	GPM_PREF_DIR "/session_request_save"
+
 typedef enum {
 	GPM_ICON_POLICY_ALWAYS,
 	GPM_ICON_POLICY_PRESENT,
