@@ -24,8 +24,6 @@
 #  include <config.h>
 #endif
 
-#define USE_EGGTRAYICON		(GTK_MINOR_VERSION < 9)
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
@@ -40,18 +38,15 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-
-#if ! USE_EGGTRAYICON
 #include <gtk/gtkstatusicon.h>
-#endif
 
-#include <libgnomeui/gnome-help.h> /* for gnome_help_display */
+#include <libgnomeui/gnome-help.h>
 
 #ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
 #endif
 
-#include "gpm-common.h" /* For GPM_HOMEPAGE_URL, etc */
+#include "gpm-common.h"
 #include "gpm-stock-icons.h"
 #include "gpm-tray-icon.h"
 #include "gpm-debug.h"
@@ -67,13 +62,7 @@ struct GpmTrayIconPrivate
 	GtkUIManager	*ui_manager;
 	GtkActionGroup	*actiongroup;
 	GtkWidget	*popup_menu;
-#if USE_EGGTRAYICON
-	GtkTooltips	*tooltips;
-	GtkWidget	*image;
-	GtkWidget	*ebox;
-#else
 	GtkStatusIcon	*status_icon;
-#endif
 
 	gboolean	 show_notifications;
 	gboolean	 is_visible;
@@ -81,7 +70,7 @@ struct GpmTrayIconPrivate
 	gboolean	 can_suspend;
 	gboolean	 can_hibernate;
 
-	char		*stock_id;
+	gchar		*stock_id;
 #ifdef HAVE_LIBNOTIFY
 	NotifyNotification *notify;
 #endif
@@ -103,6 +92,7 @@ static void gpm_tray_icon_suspend_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_hibernate_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_info_cb	 	(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_preferences_cb	(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_statistics_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_help_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_about_cb		(GtkAction *action, GpmTrayIcon *icon);
 
@@ -116,6 +106,8 @@ static GtkActionEntry gpm_tray_icon_action_entries [] =
 	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_preferences_cb) },
 	{ "TrayInfo", GTK_STOCK_DIALOG_INFO, N_("_Information"),
 	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_info_cb) },
+	{ "TrayStatistics", GTK_STOCK_CONVERT, N_("_Statistics"),
+	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_statistics_cb) },
 	{ "TrayHelp", GTK_STOCK_HELP, N_("_Help"), NULL,
 	  NULL, G_CALLBACK (gpm_tray_icon_show_help_cb) },
 	{ "TrayAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
@@ -125,7 +117,7 @@ static guint gpm_tray_icon_n_action_entries = G_N_ELEMENTS (gpm_tray_icon_action
 
 static guint	 signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, EGG_TYPE_TRAY_ICON)
+G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
 
 /**
  * gpm_tray_icon_enable_suspend:
@@ -144,8 +136,7 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 		icon->priv->can_suspend = enabled;
 		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TraySuspend");
-		gtk_action_set_visible (GTK_ACTION (action),
-					enabled);
+		gtk_action_set_visible (GTK_ACTION (action), enabled);
 	}
 }
 
@@ -166,9 +157,7 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 		icon->priv->can_hibernate = enabled;
 		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TrayHibernate");
-		gtk_action_set_visible (GTK_ACTION (action),
-					enabled);
-
+		gtk_action_set_visible (GTK_ACTION (action), enabled);
 	}
 }
 
@@ -178,20 +167,14 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
  * @tooltip: The tooltip text, e.g. "Batteries fully charged"
  **/
 void
-gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
-			   const char  *tooltip)
+gpm_tray_icon_set_tooltip (GpmTrayIcon  *icon,
+			   const gchar  *tooltip)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 	g_return_if_fail (tooltip != NULL);
 
-#if USE_EGGTRAYICON
-	gtk_tooltips_set_tip (icon->priv->tooltips,
-			      GTK_WIDGET (icon),
-			      tooltip, NULL);
-#else
 	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon->priv->status_icon),
 				     tooltip);
-#endif
 }
 
 /**
@@ -203,7 +186,7 @@ gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
  **/
 void
 gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
-				    const char  *stock_id)
+				    const gchar *stock_id)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
@@ -211,28 +194,15 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 		/* we only set a new icon if the name differs */
 		if (strcmp (icon->priv->stock_id, stock_id) != 0) {
 			gpm_debug ("Setting icon to %s", stock_id);
-#if USE_EGGTRAYICON
-			gtk_image_set_from_icon_name (GTK_IMAGE (icon->priv->image),
-						      stock_id,
-						      GTK_ICON_SIZE_LARGE_TOOLBAR);
-#else
 			gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (icon->priv->status_icon), stock_id);
 			gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), TRUE);
-#endif
 			/* don't keep trying to set the same icon */
 		        g_free (icon->priv->stock_id);
 			icon->priv->stock_id = g_strdup (stock_id);
 		}
 	} else {
 		/* get rid of the icon */
-#if USE_EGGTRAYICON
-		gtk_image_clear (GTK_IMAGE (icon->priv->image));
-		if (GTK_WIDGET_VISIBLE (icon->priv->image)) {
-			gtk_widget_queue_resize (GTK_WIDGET (icon->priv->image));
-		}
-#else
 		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), FALSE);
-#endif
 	}
 }
 
@@ -275,6 +245,22 @@ gpm_tray_icon_suspend_cb (GtkAction   *action,
 	g_signal_emit (icon, signals [SUSPEND], 0);
 }
 
+/**
+ * gpm_tray_icon_show_statistics_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
+static void
+gpm_tray_icon_show_statistics_cb (GtkAction   *action,
+				   GpmTrayIcon *icon)
+{
+	const gchar *command = "gnome-power-statistics";
+
+	if (! g_spawn_command_line_async (command, NULL)) {
+		gpm_warning ("Couldn't execute command: %s", command);
+	}
+}
+
 /**
  * gpm_tray_icon_show_preferences_cb:
  * @action: A valid GtkAction
@@ -284,7 +270,7 @@ static void
 gpm_tray_icon_show_preferences_cb (GtkAction   *action,
 				   GpmTrayIcon *icon)
 {
-	const char *command = "gnome-power-preferences";
+	const gchar *command = "gnome-power-preferences";
 
 	if (! g_spawn_command_line_async (command, NULL)) {
 		gpm_warning ("Couldn't execute command: %s", command);
@@ -370,49 +356,9 @@ gpm_tray_icon_show_about_cb (GtkAction   *action,
 			       "translator-credits", translators,
 			       "logo-icon-name", GPM_STOCK_APP_ICON,
 			       NULL);
-
 	g_free (license_trans);
 }
 
-#if USE_EGGTRAYICON
-/**
- * tray_popup_position_menu:
- *
- * Popup the drop-down menu at the base of the icon
- **/
-static void
-tray_popup_position_menu (GtkMenu  *menu,
-			  int      *x,
-			  int      *y,
-			  gboolean *push_in,
-			  gpointer  user_data)
-{
-	GtkWidget     *widget;
-	GtkRequisition requisition;
-	gint	       menu_xpos;
-	gint	       menu_ypos;
-
-	widget = GTK_WIDGET (user_data);
-
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
-
-	gdk_window_get_origin (widget->window, &menu_xpos, &menu_ypos);
-
-	menu_xpos += widget->allocation.x;
-	menu_ypos += widget->allocation.y;
-
-	if (menu_ypos > gdk_screen_get_height (gtk_widget_get_screen (widget)) / 2) {
-		menu_ypos -= (requisition.height + 1);
-	} else {
-		menu_ypos += widget->allocation.height + 1;
-	}
-
-	*x = menu_xpos;
-	*y = menu_ypos;
-	*push_in = TRUE;
-}
-#endif
-
 /**
  * gpm_tray_icon_popup_cleared_cd:
  * @widget: The popup Gtkwidget
@@ -425,44 +371,8 @@ gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 				GpmTrayIcon *icon)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-#if USE_EGGTRAYICON
-	/* we enable the tooltip as the menu has gone */
-	gtk_tooltips_enable (icon->priv->tooltips);
-#endif
 }
 
-#if USE_EGGTRAYICON
-/**
- * gpm_tray_icon_button_press_cb:
- * @widget: The tray icon widget
- * @event: Valid event
- * @icon: This TrayIcon class instance
- *
- * What do do when the button is left, right, middle clicked etc.
- **/
-static gboolean
-gpm_tray_icon_button_press_cb (GtkWidget      *widget,
-			       GdkEventButton *event,
-			       GpmTrayIcon    *icon)
-{
-	GtkWidget *popup;
-
-	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
-					   "/GpmTrayPopup");
-	gtk_menu_set_screen (GTK_MENU (popup),
-			     gtk_widget_get_screen (GTK_WIDGET (icon)));
-
-	/* we disable the tooltip so it doesn't clash with the menu. See #331075 */
-	gtk_tooltips_disable (icon->priv->tooltips);
-	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
-			tray_popup_position_menu, widget,
-			2,
-			gtk_get_current_event_time ());
-	return TRUE;
-}
-#endif
-
 /**
  * gpm_tray_icon_sync_actions:
  * @icon: This TrayIcon class instance
@@ -527,6 +437,7 @@ gpm_tray_icon_constructor (GType		  type,
 					   "    <separator />"
 					   "    <menuitem action=\"TrayPreferences\" />"
 					   "    <menuitem action=\"TrayInfo\" />"
+/*					   "    <menuitem action=\"TrayStatistics\" />"*/
 					   "    <separator />"
 					   "    <menuitem action=\"TrayHelp\" />"
 					   "    <menuitem action=\"TrayAbout\" />"
@@ -606,23 +517,14 @@ gpm_tray_icon_show (GpmTrayIcon *icon,
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
 	if (enabled) {
-#if USE_EGGTRAYICON
-		gtk_widget_show_all (GTK_WIDGET (icon));
-#else
 		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), TRUE);
-#endif
 		icon->priv->is_visible = TRUE;
 	} else {
-#if USE_EGGTRAYICON
-		gtk_widget_hide_all (GTK_WIDGET (icon));
-#else
 		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), FALSE);
-#endif
 		icon->priv->is_visible = FALSE;
 	}
 }
 
-#if !USE_EGGTRAYICON
 /**
  * gpm_tray_icon_popup_menu_cb:
  * @button: Which buttons are pressed
@@ -639,8 +541,6 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 	GtkWidget *popup;
 	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
 					   "/GpmTrayPopup");
-	gtk_menu_set_screen (GTK_MENU (popup),
-			     gtk_widget_get_screen (GTK_WIDGET (icon)));
 
 	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
 			gtk_status_icon_position_menu, status_icon,
@@ -658,9 +558,8 @@ static void
 gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 			   GpmTrayIcon   *icon)
 {
-	g_debug ("icon activated");
+	gpm_debug ("icon activated");
 }
-#endif
 
 /**
  * gpm_tray_icon_init:
@@ -681,19 +580,6 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 
 	icon->priv->ui_manager = gtk_ui_manager_new ();
 
-#if USE_EGGTRAYICON
-	icon->priv->tooltips = gtk_tooltips_new ();
-	icon->priv->ebox = gtk_event_box_new ();
-	g_signal_connect_object (G_OBJECT (icon->priv->ebox),
-				 "button_press_event",
-				 G_CALLBACK (gpm_tray_icon_button_press_cb),
-				 icon, 0);
-
-	icon->priv->image = gtk_image_new ();
-	gtk_container_add (GTK_CONTAINER (icon->priv->ebox), icon->priv->image);
-
-	gtk_container_add (GTK_CONTAINER (icon), icon->priv->ebox);
-#else
 	icon->priv->status_icon = gtk_status_icon_new ();
 	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
 				 "popup_menu",
@@ -703,7 +589,6 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 				 "activate",
 				 G_CALLBACK (gpm_tray_icon_activate_cb),
 				 icon, 0);
-#endif
 
 	gpm_tray_icon_show (GPM_TRAY_ICON (icon), FALSE);
 
@@ -731,10 +616,6 @@ gpm_tray_icon_finalize (GObject *object)
 
 	g_return_if_fail (tray_icon->priv != NULL);
 
-#if USE_EGGTRAYICON
-	gtk_object_destroy (GTK_OBJECT (tray_icon->priv->tooltips));
-#endif
-
 	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
 }
 
@@ -780,26 +661,25 @@ notification_closed_cb (NotifyNotification *notify,
  **/
 static gboolean
 libnotify_event (GpmTrayIcon    *icon,
-		 const char	*title,
-		 const char	*content,
+		 const gchar	*title,
+		 const gchar	*content,
 		 guint		 timeout,
-		 const char	*msgicon,
+		 const gchar	*msgicon,
 		 GpmNotifyLevel	 urgency)
 {
-	GtkWidget *point = NULL;
-
 	if (icon->priv->notify != NULL) {
 		notify_notification_close (icon->priv->notify, NULL);
 	}
 
-	/* Point to the center of the icon as per the GNOME HIG, #338638 */
-#if USE_EGGTRAYICON
-	if (icon->priv->is_visible) {
-		point = icon->priv->image;
-	}
-#endif
+#if 0
+	/* we can't point because of a bug in libnotify. Need dependency on 0.4.3 */
+	icon->priv->notify = notify_notification_new_with_status_icon (title, content,
+								       msgicon, icon->priv->status_icon);
+#else
+	/* Use the bottom of the screen */
 	icon->priv->notify = notify_notification_new (title, content,
-						      msgicon, point);
+						      msgicon, NULL);
+#endif
 
 	notify_notification_set_timeout (icon->priv->notify, timeout * 1000);
 
@@ -835,10 +715,10 @@ libnotify_event (GpmTrayIcon    *icon,
  **/
 static gboolean
 libnotify_event (GpmTrayIcon    *icon,
-		 const char	*title,
-		 const char	*content,
+		 const gchar	*title,
+		 const gchar	*content,
 		 guint		 timeout,
-		 const char	*msgicon,
+		 const gchar	*msgicon,
 		 GpmNotifyLevel	 urgency)
 {
 	GtkWidget     *dialog;
@@ -883,10 +763,10 @@ libnotify_event (GpmTrayIcon    *icon,
  **/
 void
 gpm_tray_icon_notify (GpmTrayIcon	*icon,
-		      const char	*title,
-		      const char	*content,
+		      const gchar	*title,
+		      const gchar	*content,
 		      guint		 timeout,
-		      const char	*msgicon,
+		      const gchar	*msgicon,
 		      GpmNotifyLevel	 urgency)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
